'''OpenGL extension NV.texture_npot_2D_mipmap

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.texture_npot_2D_mipmap to provide a more 
Python-friendly API

Overview (from the spec)
	
	Conventional OpenGL ES 2.0 allows the use of non-power-of-two (NPOT)
	textures with the limitation that mipmap minification filters can
	not be used. This extension relaxes this restriction and adds
	limited mipmap support for 2D NPOT textures.
	
	With this extension, NPOT textures are specified and applied
	identically to mipmapped power-of-two 2D textures with the following
	limitations:
	
	  - The texture wrap modes must be CLAMP_TO_EDGE.
	
	  - Coordinates used for texture sampling on an NPOT texture using a
	    mipmapped minification filter must lie within the range [0,1].
	    Coordinate clamping is not performed by the GL in this case,
	    causing values outside this range to produce undefined results.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_npot_2D_mipmap.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.texture_npot_2D_mipmap import *
from OpenGL.raw.GLES2.NV.texture_npot_2D_mipmap import _EXTENSION_NAME

def glInitTextureNpot2DMipmapNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_EXT_yuv_surface'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_EXT_yuv_surface',error_checker=_errors._error_checker)
EGL_YUV_BUFFER_EXT=_C('EGL_YUV_BUFFER_EXT',0x3300)
EGL_YUV_CSC_STANDARD_2020_EXT=_C('EGL_YUV_CSC_STANDARD_2020_EXT',0x330D)
EGL_YUV_CSC_STANDARD_601_EXT=_C('EGL_YUV_CSC_STANDARD_601_EXT',0x330B)
EGL_YUV_CSC_STANDARD_709_EXT=_C('EGL_YUV_CSC_STANDARD_709_EXT',0x330C)
EGL_YUV_CSC_STANDARD_EXT=_C('EGL_YUV_CSC_STANDARD_EXT',0x330A)
EGL_YUV_DEPTH_RANGE_EXT=_C('EGL_YUV_DEPTH_RANGE_EXT',0x3317)
EGL_YUV_DEPTH_RANGE_FULL_EXT=_C('EGL_YUV_DEPTH_RANGE_FULL_EXT',0x3319)
EGL_YUV_DEPTH_RANGE_LIMITED_EXT=_C('EGL_YUV_DEPTH_RANGE_LIMITED_EXT',0x3318)
EGL_YUV_NUMBER_OF_PLANES_EXT=_C('EGL_YUV_NUMBER_OF_PLANES_EXT',0x3311)
EGL_YUV_ORDER_AYUV_EXT=_C('EGL_YUV_ORDER_AYUV_EXT',0x3308)
EGL_YUV_ORDER_EXT=_C('EGL_YUV_ORDER_EXT',0x3301)
EGL_YUV_ORDER_UYVY_EXT=_C('EGL_YUV_ORDER_UYVY_EXT',0x3305)
EGL_YUV_ORDER_VYUY_EXT=_C('EGL_YUV_ORDER_VYUY_EXT',0x3307)
EGL_YUV_ORDER_YUV_EXT=_C('EGL_YUV_ORDER_YUV_EXT',0x3302)
EGL_YUV_ORDER_YUYV_EXT=_C('EGL_YUV_ORDER_YUYV_EXT',0x3304)
EGL_YUV_ORDER_YVU_EXT=_C('EGL_YUV_ORDER_YVU_EXT',0x3303)
EGL_YUV_ORDER_YVYU_EXT=_C('EGL_YUV_ORDER_YVYU_EXT',0x3306)
EGL_YUV_PLANE_BPP_0_EXT=_C('EGL_YUV_PLANE_BPP_0_EXT',0x331B)
EGL_YUV_PLANE_BPP_10_EXT=_C('EGL_YUV_PLANE_BPP_10_EXT',0x331D)
EGL_YUV_PLANE_BPP_8_EXT=_C('EGL_YUV_PLANE_BPP_8_EXT',0x331C)
EGL_YUV_PLANE_BPP_EXT=_C('EGL_YUV_PLANE_BPP_EXT',0x331A)
EGL_YUV_SUBSAMPLE_4_2_0_EXT=_C('EGL_YUV_SUBSAMPLE_4_2_0_EXT',0x3313)
EGL_YUV_SUBSAMPLE_4_2_2_EXT=_C('EGL_YUV_SUBSAMPLE_4_2_2_EXT',0x3314)
EGL_YUV_SUBSAMPLE_4_4_4_EXT=_C('EGL_YUV_SUBSAMPLE_4_4_4_EXT',0x3315)
EGL_YUV_SUBSAMPLE_EXT=_C('EGL_YUV_SUBSAMPLE_EXT',0x3312)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_histogram'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_histogram',error_checker=_errors._error_checker)
GL_HISTOGRAM_ALPHA_SIZE_EXT=_C('GL_HISTOGRAM_ALPHA_SIZE_EXT',0x802B)
GL_HISTOGRAM_BLUE_SIZE_EXT=_C('GL_HISTOGRAM_BLUE_SIZE_EXT',0x802A)
GL_HISTOGRAM_EXT=_C('GL_HISTOGRAM_EXT',0x8024)
GL_HISTOGRAM_FORMAT_EXT=_C('GL_HISTOGRAM_FORMAT_EXT',0x8027)
GL_HISTOGRAM_GREEN_SIZE_EXT=_C('GL_HISTOGRAM_GREEN_SIZE_EXT',0x8029)
GL_HISTOGRAM_LUMINANCE_SIZE_EXT=_C('GL_HISTOGRAM_LUMINANCE_SIZE_EXT',0x802C)
GL_HISTOGRAM_RED_SIZE_EXT=_C('GL_HISTOGRAM_RED_SIZE_EXT',0x8028)
GL_HISTOGRAM_SINK_EXT=_C('GL_HISTOGRAM_SINK_EXT',0x802D)
GL_HISTOGRAM_WIDTH_EXT=_C('GL_HISTOGRAM_WIDTH_EXT',0x8026)
GL_MINMAX_EXT=_C('GL_MINMAX_EXT',0x802E)
GL_MINMAX_FORMAT_EXT=_C('GL_MINMAX_FORMAT_EXT',0x802F)
GL_MINMAX_SINK_EXT=_C('GL_MINMAX_SINK_EXT',0x8030)
GL_PROXY_HISTOGRAM_EXT=_C('GL_PROXY_HISTOGRAM_EXT',0x8025)
GL_TABLE_TOO_LARGE_EXT=_C('GL_TABLE_TOO_LARGE_EXT',0x8031)
@_f
@_p.types(None,_cs.GLenum,_cs.GLboolean,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetHistogramEXT(target,reset,format,type,values):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetHistogramParameterfvEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetHistogramParameterivEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLboolean,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetMinmaxEXT(target,reset,format,type,values):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetMinmaxParameterfvEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetMinmaxParameterivEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLboolean)
def glHistogramEXT(target,width,internalformat,sink):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLboolean)
def glMinmaxEXT(target,internalformat,sink):pass
@_f
@_p.types(None,_cs.GLenum)
def glResetHistogramEXT(target):pass
@_f
@_p.types(None,_cs.GLenum)
def glResetMinmaxEXT(target):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_conservative_raster_dilate'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_conservative_raster_dilate',error_checker=_errors._error_checker)
GL_CONSERVATIVE_RASTER_DILATE_GRANULARITY_NV=_C('GL_CONSERVATIVE_RASTER_DILATE_GRANULARITY_NV',0x937B)
GL_CONSERVATIVE_RASTER_DILATE_NV=_C('GL_CONSERVATIVE_RASTER_DILATE_NV',0x9379)
GL_CONSERVATIVE_RASTER_DILATE_RANGE_NV=_C('GL_CONSERVATIVE_RASTER_DILATE_RANGE_NV',0x937A)
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat)
def glConservativeRasterParameterfNV(pname,value):pass



'''OpenGL extension ARB.spirv_extensions

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.spirv_extensions to provide a more 
Python-friendly API

Overview (from the spec)
	
	ARB_gl_spirv added support for using SPIR-V modules in OpenGL.
	However it only added support for SPIR-V 1.0 concepts that were part of
	the OpenGL 4.5 Core Profile.
	
	There are a great number of additional OpenGL ARB and vendor extensions
	which add shading language concepts and since they were defined prior
	to the existence of SPIR-V support in OpenGL they don't add SPIR-V support
	for their additional features.  Ideally GL_ARB_gl_spirv would have added
	support for them, but as noted in Issue 27 of that extension, support for
	them was left as a future exercise.
	
	Now that at least some of that functionality has been defined via SPIR-V
	extensions, there is currently no way for an OpenGL implementation to
	advertise that is supports additional SPIR-V extensions.
	
	This extension provides a mechanism for an implementation to advertise
	which SPIR-V extensions it supports, and further provides a place where
	the SPIR-V environment for those extensions can be documented for OpenGL.
	
	It is expected that this document can be extended over time as SPIR-V
	support for additional extensions is added. The mapping between GLSL and
	SPIR-V concepts and any other pertinent information can be provided here
	as interactions with the corresponding OpenGL and SPIR-V extensions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/spirv_extensions.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.spirv_extensions import *
from OpenGL.raw.GL.ARB.spirv_extensions import _EXTENSION_NAME

def glInitSpirvExtensionsARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""OpenGL.GL, the core GL library and extensions to it"""
# early import of our modules to prevent import loops...
from OpenGL import error as _error
from OpenGL.GL.VERSION.GL_1_1 import *
from OpenGL.GL.pointers import *
from OpenGL.GL.images import *

from OpenGL.GL.exceptional import *

from OpenGL.GL.glget import *

from OpenGL.GL.VERSION.GL_1_2 import *
from OpenGL.GL.VERSION.GL_1_3 import *
from OpenGL.GL.VERSION.GL_1_4 import *
from OpenGL.GL.VERSION.GL_1_5 import *
from OpenGL.GL.VERSION.GL_2_0 import *
from OpenGL.GL.VERSION.GL_2_1 import *
from OpenGL.GL.VERSION.GL_3_0 import *
from OpenGL.GL.VERSION.GL_3_1 import *
from OpenGL.GL.VERSION.GL_3_2 import *
from OpenGL.GL.VERSION.GL_3_3 import *
from OpenGL.GL.VERSION.GL_4_0 import *
from OpenGL.GL.VERSION.GL_4_1 import *
from OpenGL.GL.VERSION.GL_4_2 import *
from OpenGL.GL.VERSION.GL_4_3 import *
from OpenGL.GL.VERSION.GL_4_4 import *
from OpenGL.GL.VERSION.GL_4_5 import *
from OpenGL.GL.VERSION.GL_4_6 import *

from OpenGL.error import *
GLerror = GLError

# Now the aliases...
glRotate = glRotated
glTranslate = glTranslated
glLight = glLightfv
glTexCoord = glTexCoord2d
glScale = glScaled
#glColor = glColor3f
glNormal = glNormal3d

glGetBoolean = glGetBooleanv
glGetDouble = glGetDoublev
glGetFloat = glGetFloatv
glGetInteger = glGetIntegerv 
glGetPolygonStippleub = glGetPolygonStipple

from OpenGL.GL import vboimplementation as _core_implementation
from OpenGL.GL.ARB import vboimplementation as _arb_implementation



'''OpenGL extension NV.path_rendering

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.path_rendering to provide a more 
Python-friendly API

Overview (from the spec)
	
	Conventional OpenGL supports rendering images (pixel rectangles and
	bitmaps) and simple geometric primitives (points, lines, polygons).
	
	This extension adds a new rendering paradigm, known as path rendering,
	for rendering filled and stroked paths.  Path rendering is not novel
	but rather a standard part of most resolution-independent 2D rendering
	systems such as Flash, PDF, Silverlight, SVG, Java 2D, Office
	drawings, TrueType fonts, PostScript and its fonts, Quartz 2D, XML
	Paper Specification (XPS), and OpenVG.  What is novel is the ability
	to mix path rendering with arbitrary OpenGL 3D rendering and imaging.
	
	With this extension, path rendering becomes a first-class rendering
	mode within the OpenGL graphics system that can be arbitrarily mixed
	with existing OpenGL rendering and can take advantage of OpenGL's
	existing mechanisms for texturing, programmability, and per-fragment
	operations.
	
	Unlike geometric primitive rendering, paths are specified on a 2D
	(non-projective) plane rather than in 3D (projective) space.
	Even though the path is defined in a 2D plane, every path can
	be transformed into 3D clip space allowing for 3D view frustum &
	user-defined clipping, depth offset, and depth testing in the same
	manner as geometric primitive rendering.
	
	Both geometric primitive rendering and path rendering support
	rasterization of edges defined by line segments; however, path
	rendering also allows path segments to be specified by Bezier (cubic
	or quadratic) curves or partial elliptical arcs.  This allows path
	rendering to define truly curved primitive boundaries unlike the
	straight edges of line and polygon primitives.  Whereas geometric
	primitive rendering requires convex polygons for well-defined
	rendering results, path rendering allows (and encourages!) concave
	and curved outlines to be specified.  These paths are even allowed
	to self-intersect.
	
	When filling closed paths, the winding of paths (counterclockwise
	or clockwise) determines whether pixels are inside or outside of
	the path.
	
	Paths can also be stroked whereby, conceptually, a fixed-width "brush"
	is pulled along the path such that the brush remains orthogonal to
	the gradient of each path segment.  Samples within the sweep of this
	brush are considered inside the stroke of the path.
	
	This extension supports path rendering through a sequence of three
	operations:
	
	    1.  Path specification is the process of creating and updating
	        a path object consisting of a set of path commands and a
	        corresponding set of 2D vertices.
	
	        Path commands can be specified explicitly from path command
	        and coordinate data, parsed from a string based on standard
	        grammars for representing paths, or specified by a particular
	        glyph of standard font representations.  Also new paths can
	        be specified by weighting one or more existing paths so long
	        as all the weighted paths have consistent command sequences.
	
	        Each path object contains zero or more subpaths specified
	        by a sequence of line segments, partial elliptical arcs,
	        and (cubic or quadratic) Bezier curve segments.  Each path
	        may contain multiple subpaths that can be closed (forming
	        a contour) or open.
	
	    2.  Path stenciling is the process of updating the stencil buffer
	        based on a path's coverage transformed into window space.
	
	        Path stenciling can determine either the filled or stroked
	        coverage of a path.
	
	        The details of path stenciling are explained within the core
	        of the specification.
	
	        Stenciling a stroked path supports all the standard
	        embellishments for path stroking such as end caps, join
	        styles, miter limits, dashing, and dash caps.  These stroking
	        properties specified are parameters of path objects.
	
	    3.  Path covering is the process of emitting simple (convex &
	        planar) geometry that (conservatively) "covers" the path's
	        sample coverage in the stencil buffer.  During path covering,
	        stencil testing can be configured to discard fragments not
	        within the actual coverage of the path as determined by
	        prior path stenciling.
	
	        Path covering can cover either the filled or stroked coverage
	        of a path.
	
	        The details of path covering are explained within the core
	        of the specification.
	
	To render a path object into the color buffer, an application specifies
	a path object and then uses a two-step rendering process.  First, the
	path object is stenciled whereby the path object's stroked or filled
	coverage is rasterized into the stencil buffer.  Second, the path object
	is covered whereby conservative bounding geometry for the path is
	transformed and rasterized with stencil testing configured to test against
	the coverage information written to the stencil buffer in the first step
	so that only fragments covered by the path are written during this second
	step.  Also during this second step written pixels typically have
	their stencil value reset (so there's no need for clearing the
	stencil buffer between rendering each path).
	
	Here is an example of specifying and then rendering a five-point
	star and a heart as a path using Scalable Vector Graphics (SVG)
	path description syntax:
	
	    GLuint pathObj = 42;
	    const char *svgPathString =
	      // star
	      "M100,180 L40,10 L190,120 L10,120 L160,10 z"
	      // heart
	      "M300 300 C 100 400,100 200,300 100,500 200,500 400,300 300Z";
	    glPathStringNV(pathObj, GL_PATH_FORMAT_SVG_NV,
	                   (GLsizei)strlen(svgPathString), svgPathString);
	
	Alternatively applications oriented around the PostScript imaging
	model can use the PostScript user path syntax instead:
	
	    const char *psPathString =
	      // star
	      "100 180 moveto"
	      " 40 10 lineto 190 120 lineto 10 120 lineto 160 10 lineto closepath"
	      // heart
	      " 300 300 moveto"
	      " 100 400 100 200 300 100 curveto"
	      " 500 200 500 400 300 300 curveto closepath";
	    glPathStringNV(pathObj, GL_PATH_FORMAT_PS_NV,
	                   (GLsizei)strlen(psPathString), psPathString);
	
	The PostScript path syntax also supports compact and precise binary
	encoding and includes PostScript-style circular arcs.
	
	Or the path's command and coordinates can be specified explicitly:
	
	    static const GLubyte pathCommands[10] =
	      { GL_MOVE_TO_NV, GL_LINE_TO_NV, GL_LINE_TO_NV, GL_LINE_TO_NV,
	        GL_LINE_TO_NV, GL_CLOSE_PATH_NV,
	        'M', 'C', 'C', 'Z' };  // character aliases
	    static const GLshort pathCoords[12][2] =
	      { {100, 180}, {40, 10}, {190, 120}, {10, 120}, {160, 10},
	        {300,300}, {100,400}, {100,200}, {300,100},
	        {500,200}, {500,400}, {300,300} };
	    glPathCommandsNV(pathObj, 10, pathCommands, 24, GL_SHORT, pathCoords);
	
	Before rendering to a window with a stencil buffer, clear the stencil
	buffer to zero and the color buffer to black:
	
	    glClearStencil(0);
	    glClearColor(0,0,0,0);
	    glStencilMask(~0);
	    glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	Use an orthographic path-to-clip-space transform to map the
	[0..500]x[0..400] range of the star's path coordinates to the [-1..1]
	clip space cube:
	
	    glMatrixLoadIdentityEXT(GL_PROJECTION);
	    glMatrixLoadIdentityEXT(GL_MODELVIEW);
	    glMatrixOrthoEXT(GL_MODELVIEW, 0, 500, 0, 400, -1, 1);
	
	Stencil the path:
	
	    glStencilFillPathNV(pathObj, GL_COUNT_UP_NV, 0x1F);
	
	The 0x1F mask means the counting uses modulo-32 arithmetic. In
	principle the star's path is simple enough (having a maximum winding
	number of 2) that modulo-4 arithmetic would be sufficient so the mask
	could be 0x3.  Or a mask of all 1's (~0) could be used to count with
	all available stencil bits.
	
	Now that the coverage of the star and the heart have been rasterized
	into the stencil buffer, cover the path with a non-zero fill style
	(indicated by the GL_NOTEQUAL stencil function with a zero reference
	value):
	
	    glEnable(GL_STENCIL_TEST);
	    glStencilFunc(GL_NOTEQUAL, 0, 0x1F);
	    glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
	    glColor3f(1,1,0); // yellow
	    glCoverFillPathNV(pathObj, GL_BOUNDING_BOX_NV);
	
	The result is a yellow star (with a filled center) to the left of
	a yellow heart.
	
	The GL_ZERO stencil operation ensures that any covered samples
	(meaning those with non-zero stencil values) are zero'ed when
	the path cover is rasterized. This allows subsequent paths to be
	rendered without clearing the stencil buffer again.
	
	A similar two-step rendering process can draw a white outline
	over the star and heart.
	
	Before rendering, configure the path object with desirable path
	parameters for stroking.  Specify a wider 6.5-unit stroke and
	the round join style:
	
	    glPathParameteriNV(pathObj, GL_PATH_JOIN_STYLE_NV, GL_ROUND_NV);
	    glPathParameterfNV(pathObj, GL_PATH_STROKE_WIDTH_NV, 6.5);
	
	 Now stencil the path's stroked coverage into the stencil buffer,
	 setting the stencil to 0x1 for all stencil samples within the
	 transformed path.
	
	    glStencilStrokePathNV(pathObj, 0x1, ~0);
	
	 Cover the path's stroked coverage (with a hull this time instead
	 of a bounding box; the choice doesn't really matter here) while
	 stencil testing that writes white to the color buffer and again
	 zero the stencil buffer.
	
	    glColor3f(1,1,1); // white
	    glCoverStrokePathNV(pathObj, GL_CONVEX_HULL_NV);
	
	 In this example, constant color shading is used but the application
	 can specify their own arbitrary shading and/or blending operations,
	 whether with Cg compiled to fragment program assembly, GLSL, or
	 fixed-function fragment processing.
	
	 More complex path rendering is possible such as clipping one path to
	 another arbitrary path.  This is because stencil testing (as well
	 as depth testing, depth bound test, clip planes, and scissoring)
	 can restrict path stenciling.
	
	 Now let's render the word "OpenGL" atop the star and heart.
	
	 First create a sequence of path objects for the glyphs for the
	 characters in "OpenGL":
	
	    GLuint glyphBase = glGenPathsNV(6);
	    const unsigned char *word = "OpenGL";
	    const GLsizei wordLen = (GLsizei)strlen(word);
	    const GLfloat emScale = 2048;  // match TrueType convention
	    GLuint templatePathObject = ~0;  // Non-existent path object
	    glPathGlyphsNV(glyphBase,
	                   GL_SYSTEM_FONT_NAME_NV, "Helvetica", GL_BOLD_BIT_NV,
	                   wordLen, GL_UNSIGNED_BYTE, word,
	                   GL_SKIP_MISSING_GLYPH_NV, ~0, emScale);
	    glPathGlyphsNV(glyphBase,
	                   GL_SYSTEM_FONT_NAME_NV, "Arial", GL_BOLD_BIT_NV,
	                   wordLen, GL_UNSIGNED_BYTE, word,
	                   GL_SKIP_MISSING_GLYPH_NV, ~0, emScale);
	    glPathGlyphsNV(glyphBase,
	                   GL_STANDARD_FONT_NAME_NV, "Sans", GL_BOLD_BIT_NV,
	                   wordLen, GL_UNSIGNED_BYTE, word,
	                   GL_USE_MISSING_GLYPH_NV, ~0, emScale);
	
	Glyphs are loaded for three different fonts in priority order:
	Helvetica first, then Arial, and if neither of those loads, use the
	standard sans-serif font.  If a prior glPathGlyphsNV is successful
	and specifies the path object range, the subsequent glPathGlyphsNV
	commands silently avoid re-specifying the already existent path
	objects.
	
	Now query the (kerned) separations for the word "OpenGL" and build
	a set of horizontal translations advancing each successive glyph by
	its kerning distance with the following glyph.
	
	    GLfloat xtranslate[6+1];  // wordLen+1
	    glGetPathSpacingNV(GL_ACCUM_ADJACENT_PAIRS_NV,
	                       wordLen+1, GL_UNSIGNED_BYTE,
	                       "\000\001\002\003\004\005\005",  // repeat last letter twice
	                       glyphBase,
	                       1.0f, 1.0f,
	                       GL_TRANSLATE_X_NV,
	                       xtranslate);
	
	Next determine the font-wide vertical minimum and maximum for the
	font face by querying the per-font metrics of any one of the glyphs
	from the font face.
	
	    GLfloat yMinMax[2];
	    glGetPathMetricRangeNV(GL_FONT_Y_MIN_BOUNDS_BIT_NV|GL_FONT_Y_MAX_BOUNDS_BIT_NV,
	                           glyphBase, /*count*/1,
	                           2*sizeof(GLfloat),
	                           yMinMax);
	
	Use an orthographic path-to-clip-space transform to map the
	word's bounds to the [-1..1] clip space cube:
	
	    glMatrixLoadIdentityEXT(GL_PROJECTION);
	    glMatrixOrthoEXT(GL_MODELVIEW,
	                     0, xtranslate[6], yMinMax[0], yMinMax[1],
	                     -1, 1);
	
	Stencil the filled paths of the sequence of glyphs for "OpenGL",
	each transformed by the appropriate 2D translations for spacing.
	
	    glStencilFillPathInstancedNV(6, GL_UNSIGNED_BYTE,
	                                 "\000\001\002\003\004\005",
	                                 glyphBase,
	                                 GL_PATH_FILL_MODE_NV, 0xFF,
	                                 GL_TRANSLATE_X_NV, xtranslate);
	
	 Cover the bounding box union of the glyphs with 50% gray.
	
	    glEnable(GL_STENCIL_TEST);
	    glStencilFunc(GL_NOTEQUAL, 0, 0xFF);
	    glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
	    glColor3f(0.5,0.5,0.5); // 50% gray
	    glCoverFillPathInstancedNV(6, GL_UNSIGNED_BYTE,
	                               "\000\001\002\003\004\005",
	                               glyphBase,
	                               GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV,
	                               GL_TRANSLATE_X_NV, xtranslate);
	
	Voila, the word "OpenGL" in gray is now stenciled into the framebuffer.
	
	Instead of solid 50% gray, the cover operation can apply a linear
	gradient that changes from green (RGB=0,1,0) at the top of the word
	"OpenGL" to blue (RGB=0,0,1) at the bottom of "OpenGL":
	
	    GLfloat rgbGen[3][3] = {
	      0, 0, 0,  // red   = constant zero
	      0, 1, 0,  // green = varies with y from bottom (0) to top (1)
	      0, -1, 1  // blue  = varies with y from bottom (1) to top (0)
	    };
	    glPathColorGenNV(GL_PRIMARY_COLOR, GL_PATH_OBJECT_BOUNDING_BOX_NV,
	                     GL_RGB, &rgbGen[0][0]);
	
	Instead of loading just the glyphs for the characters in "OpenGL",
	the entire character set could be loaded.  This allows the characters
	of the string to be mapped (offset by the glyphBase) to path object names.
	A range of glyphs can be loaded like this:
	
	    const int numChars = 256;  // ISO/IEC 8859-1 8-bit character range
	    GLuint glyphBase = glGenPathsNV(numChars);
	    glPathGlyphRangeNV(glyphBase,
	                       GL_SYSTEM_FONT_NAME_NV, "Helvetica", GL_BOLD_BIT_NV,
	                       0, numChars,
	                       GL_SKIP_MISSING_GLYPH_NV, ~0, emScale);
	    glPathGlyphRangeNV(glyphBase,
	                       GL_SYSTEM_FONT_NAME_NV, "Arial", GL_BOLD_BIT_NV,
	                       0, numChars,
	                       GL_SKIP_MISSING_GLYPH_NV, ~0, emScale);
	    glPathGlyphRangeNV(glyphBase,
	                       GL_STANDARD_FONT_NAME_NV, "Sans", GL_BOLD_BIT_NV,
	                       0, numChars,
	                       GL_USE_MISSING_GLYPH_NV, ~0, emScale);
	
	Given a range of glyphs loaded as path objects, (kerned) spacing
	information can now be queried for the string:
	
	    glGetPathSpacingNV(GL_ACCUM_ADJACENT_PAIRS_NV,
	                       7, GL_UNSIGNED_BYTE, "OpenGLL", // repeat L to get final spacing
	                       glyphBase,
	                       1.0f, 1.0f,
	                       GL_TRANSLATE_X_NV,
	                       kerning);
	
	Using the range of glyphs, stenciling and covering the instanced
	paths for "OpenGL" can be done this way:
	
	    glStencilFillPathInstancedNV(6, GL_UNSIGNED_BYTE, "OpenGL",
	                                 glyphBase,
	                                 GL_PATH_FILL_MODE_NV, 0xFF,
	                                 GL_TRANSLATE_X_NV, xtranslate);
	
	    glCoverFillPathInstancedNV(6, GL_UNSIGNED_BYTE, "OpenGL",
	                               glyphBase,
	                               GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV,
	                               GL_TRANSLATE_X_NV, xtranslate);
	
	The "stencil" and "cover" steps can be combined in a single command:
	
	    glStencilThenCoverFillPathInstancedNV(6, GL_UNSIGNED_BYTE, "OpenGL",
	                                          glyphBase,
	                                          GL_PATH_FILL_MODE_NV, 0xFF,
	                                          GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV
	                                          GL_TRANSLATE_X_NV, xtranslate);
	
	XXX add path clipping example to demonstrate glPathStencilFuncNV.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/path_rendering.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.path_rendering import *
from OpenGL.raw.GLES2.NV.path_rendering import _EXTENSION_NAME

def glInitPathRenderingNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glPathCommandsNV.commands size not checked against numCommands
# INPUT glPathCommandsNV.coords size not checked against 'numCoords,coordType'
glPathCommandsNV=wrapper.wrapper(glPathCommandsNV).setInputArraySize(
    'commands', None
).setInputArraySize(
    'coords', None
)
# INPUT glPathCoordsNV.coords size not checked against 'numCoords,coordType'
glPathCoordsNV=wrapper.wrapper(glPathCoordsNV).setInputArraySize(
    'coords', None
)
# INPUT glPathSubCommandsNV.commands size not checked against numCommands
# INPUT glPathSubCommandsNV.coords size not checked against 'numCoords,coordType'
glPathSubCommandsNV=wrapper.wrapper(glPathSubCommandsNV).setInputArraySize(
    'commands', None
).setInputArraySize(
    'coords', None
)
# INPUT glPathSubCoordsNV.coords size not checked against 'numCoords,coordType'
glPathSubCoordsNV=wrapper.wrapper(glPathSubCoordsNV).setInputArraySize(
    'coords', None
)
# INPUT glPathStringNV.pathString size not checked against length
glPathStringNV=wrapper.wrapper(glPathStringNV).setInputArraySize(
    'pathString', None
)
# INPUT glPathGlyphsNV.charcodes size not checked against 'numGlyphs,type,charcodes'
# INPUT glPathGlyphsNV.fontName size not checked against 'fontTarget,fontName'
glPathGlyphsNV=wrapper.wrapper(glPathGlyphsNV).setInputArraySize(
    'charcodes', None
).setInputArraySize(
    'fontName', None
)
# INPUT glPathGlyphRangeNV.fontName size not checked against 'fontTarget,fontName'
glPathGlyphRangeNV=wrapper.wrapper(glPathGlyphRangeNV).setInputArraySize(
    'fontName', None
)
# INPUT glWeightPathsNV.paths size not checked against numPaths
# INPUT glWeightPathsNV.weights size not checked against numPaths
glWeightPathsNV=wrapper.wrapper(glWeightPathsNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'weights', None
)
# INPUT glTransformPathNV.transformValues size not checked against 'transformType'
glTransformPathNV=wrapper.wrapper(glTransformPathNV).setInputArraySize(
    'transformValues', None
)
# INPUT glPathParameterivNV.value size not checked against 'pname'
glPathParameterivNV=wrapper.wrapper(glPathParameterivNV).setInputArraySize(
    'value', None
)
# INPUT glPathParameterfvNV.value size not checked against 'pname'
glPathParameterfvNV=wrapper.wrapper(glPathParameterfvNV).setInputArraySize(
    'value', None
)
# INPUT glPathDashArrayNV.dashArray size not checked against dashCount
glPathDashArrayNV=wrapper.wrapper(glPathDashArrayNV).setInputArraySize(
    'dashArray', None
)
# INPUT glStencilFillPathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glStencilFillPathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glStencilFillPathInstancedNV=wrapper.wrapper(glStencilFillPathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glStencilStrokePathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glStencilStrokePathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glStencilStrokePathInstancedNV=wrapper.wrapper(glStencilStrokePathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glCoverFillPathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glCoverFillPathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glCoverFillPathInstancedNV=wrapper.wrapper(glCoverFillPathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glCoverStrokePathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glCoverStrokePathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glCoverStrokePathInstancedNV=wrapper.wrapper(glCoverStrokePathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
glGetPathParameterivNV=wrapper.wrapper(glGetPathParameterivNV).setOutput(
    'value',size=(4,),orPassIn=True
)
glGetPathParameterfvNV=wrapper.wrapper(glGetPathParameterfvNV).setOutput(
    'value',size=(4,),orPassIn=True
)
glGetPathCommandsNV=wrapper.wrapper(glGetPathCommandsNV).setOutput(
    'commands',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
glGetPathCoordsNV=wrapper.wrapper(glGetPathCoordsNV).setOutput(
    'coords',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
glGetPathDashArrayNV=wrapper.wrapper(glGetPathDashArrayNV).setOutput(
    'dashArray',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
# OUTPUT glGetPathMetricsNV.metrics COMPSIZE(metricQueryMask, numPaths, stride) 
# INPUT glGetPathMetricsNV.paths size not checked against 'numPaths,pathNameType,paths'
glGetPathMetricsNV=wrapper.wrapper(glGetPathMetricsNV).setInputArraySize(
    'paths', None
)
# OUTPUT glGetPathMetricRangeNV.metrics COMPSIZE(metricQueryMask, numPaths, stride) 
# INPUT glGetPathSpacingNV.paths size not checked against 'numPaths,pathNameType,paths'
# OUTPUT glGetPathSpacingNV.returnedSpacing COMPSIZE(pathListMode, numPaths) 
glGetPathSpacingNV=wrapper.wrapper(glGetPathSpacingNV).setInputArraySize(
    'paths', None
)
glPointAlongPathNV=wrapper.wrapper(glPointAlongPathNV).setOutput(
    'tangentX',size=(1,),orPassIn=True
).setOutput(
    'tangentY',size=(1,),orPassIn=True
).setOutput(
    'x',size=(1,),orPassIn=True
).setOutput(
    'y',size=(1,),orPassIn=True
)
# INPUT glPathColorGenNV.coeffs size not checked against 'genMode,colorFormat'
glPathColorGenNV=wrapper.wrapper(glPathColorGenNV).setInputArraySize(
    'coeffs', None
)
# INPUT glPathTexGenNV.coeffs size not checked against 'genMode,components'
glPathTexGenNV=wrapper.wrapper(glPathTexGenNV).setInputArraySize(
    'coeffs', None
)
glGetPathColorGenivNV=wrapper.wrapper(glGetPathColorGenivNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathColorGenfvNV=wrapper.wrapper(glGetPathColorGenfvNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathTexGenivNV=wrapper.wrapper(glGetPathTexGenivNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathTexGenfvNV=wrapper.wrapper(glGetPathTexGenfvNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glMatrixLoadTransposefEXT=wrapper.wrapper(glMatrixLoadTransposefEXT).setInputArraySize(
    'm', 16
)
glMatrixLoadTransposedEXT=wrapper.wrapper(glMatrixLoadTransposedEXT).setInputArraySize(
    'm', 16
)
glMatrixLoadfEXT=wrapper.wrapper(glMatrixLoadfEXT).setInputArraySize(
    'm', 16
)
glMatrixLoaddEXT=wrapper.wrapper(glMatrixLoaddEXT).setInputArraySize(
    'm', 16
)
glMatrixMultTransposefEXT=wrapper.wrapper(glMatrixMultTransposefEXT).setInputArraySize(
    'm', 16
)
glMatrixMultTransposedEXT=wrapper.wrapper(glMatrixMultTransposedEXT).setInputArraySize(
    'm', 16
)
glMatrixMultfEXT=wrapper.wrapper(glMatrixMultfEXT).setInputArraySize(
    'm', 16
)
glMatrixMultdEXT=wrapper.wrapper(glMatrixMultdEXT).setInputArraySize(
    'm', 16
)
### END AUTOGENERATED SECTION


'''OpenGL extension OVR.multiview

This module customises the behaviour of the 
OpenGL.raw.GL.OVR.multiview to provide a more 
Python-friendly API

Overview (from the spec)
	
	The method of stereo rendering supported in OpenGL is currently achieved by
	rendering to the two eye buffers sequentially.  This typically incurs double
	the application and driver overhead, despite the fact that the command
	streams and render states are almost identical.
	
	This extension seeks to address the inefficiency of sequential multiview
	rendering by adding a means to render to multiple elements of a 2D texture
	array simultaneously.  In multiview rendering, draw calls are instanced into
	each corresponding element of the texture array.  The vertex program uses a
	new gl_ViewID_OVR variable to compute per-view values, typically the vertex
	position and view-dependent variables like reflection.
	
	The formulation of this extension is high level in order to allow
	implementation freedom.  On existing hardware, applications and drivers can
	realize the benefits of a single scene traversal, even if all GPU work is
	fully duplicated per-view.  But future support could enable simultaneous
	rendering via multi-GPU, tile-based architectures could sort geometry into
	tiles for multiple views in a single pass, and the implementation could even
	choose to interleave at the fragment level for better texture cache
	utilization and more coherent fragment shader branching.
	
	The most obvious use case in this model is to support two simultaneous
	views: one view for each eye.  However, we also anticipate a usage where two
	views are rendered per eye, where one has a wide field of view and the other
	has a narrow one.  The nature of wide field of view planar projection is
	that the sample density can become unacceptably low in the view direction.
	By rendering two inset eye views per eye, we can get the required sample
	density in the center of projection without wasting samples, memory, and
	time by oversampling in the periphery.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OVR/multiview.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.OVR.multiview import *
from OpenGL.raw.GL.OVR.multiview import _EXTENSION_NAME

def glInitMultiviewOVR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.vdpau_interop

This module customises the behaviour of the 
OpenGL.raw.GL.NV.vdpau_interop to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows VDPAU video and output surfaces to be used
	for texturing and rendering.
	
	This allows the GL to process and display the content of video
	streams decoded using VDPAU.
	
	Alternatively, the GL may modify VDPAU surfaces in-place, and VDPAU
	may then process and/or display those surfaces itself.
	
	This allows the GL to be used to combine application user-interface
	elements with decoded video, implement custom video-processing
	algorithms, etc.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/vdpau_interop.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.vdpau_interop import *
from OpenGL.raw.GL.NV.vdpau_interop import _EXTENSION_NAME

def glInitVdpauInteropNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glVDPAURegisterVideoSurfaceNV.textureNames size not checked against numTextureNames
glVDPAURegisterVideoSurfaceNV=wrapper.wrapper(glVDPAURegisterVideoSurfaceNV).setInputArraySize(
    'textureNames', None
)
# INPUT glVDPAURegisterOutputSurfaceNV.textureNames size not checked against numTextureNames
# glVDPAURegisterOutputSurfaceNV.vdpSurface is OUTPUT without known output size
glVDPAURegisterOutputSurfaceNV=wrapper.wrapper(glVDPAURegisterOutputSurfaceNV).setInputArraySize(
    'textureNames', None
)
# glVDPAUGetSurfaceivNV.length is OUTPUT without known output size
glVDPAUGetSurfaceivNV=wrapper.wrapper(glVDPAUGetSurfaceivNV).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
# INPUT glVDPAUMapSurfacesNV.surfaces size not checked against numSurfaces
glVDPAUMapSurfacesNV=wrapper.wrapper(glVDPAUMapSurfacesNV).setInputArraySize(
    'surfaces', None
)
# INPUT glVDPAUUnmapSurfacesNV.surfaces size not checked against numSurface
glVDPAUUnmapSurfacesNV=wrapper.wrapper(glVDPAUUnmapSurfacesNV).setInputArraySize(
    'surfaces', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_stencil_operation_extended'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_stencil_operation_extended',error_checker=_errors._error_checker)
GL_REPLACE_VALUE_AMD=_C('GL_REPLACE_VALUE_AMD',0x874B)
GL_SET_AMD=_C('GL_SET_AMD',0x874A)
GL_STENCIL_BACK_OP_VALUE_AMD=_C('GL_STENCIL_BACK_OP_VALUE_AMD',0x874D)
GL_STENCIL_OP_VALUE_AMD=_C('GL_STENCIL_OP_VALUE_AMD',0x874C)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glStencilOpValueAMD(face,value):pass



'''OpenGL extension ATI.text_fragment_shader

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.text_fragment_shader to provide a more 
Python-friendly API

Overview (from the spec)
	
	The ATI_fragment_shader extension exposes a powerful fragment
	processing model that provides a very general means of expressing
	fragment color blending and dependent texture address modification.
	The processing is termed a fragment shader or fragment program and 
	is specifed using a register-based model in which there are fixed 
	numbers of instructions, texture lookups, read/write registers, and 
	constants.
	
	ATI_fragment_shader provides a unified instruction set
	for operating on address or color data and eliminates the
	distinction between the two.  That extension provides all the
	interfaces necessary to fully expose this programmable fragment
	processor in GL.
	
	ATI_text_fragment_shader is a redefinition of the 
	ATI_fragment_shader functionality, using a slightly different 
	interface.  The intent of creating ATI_text_fragment_shader is to
	take a step towards treating fragment programs similar to other 
	programmable parts of the GL rendering pipeline, specifically 
	vertex programs. This new interface is intended to appear 
	similar to the ARB_vertex_program API, within the limits of the 
	feature set exposed by the original ATI_fragment_shader extension.
	
	The most significant differences between the two extensions are:
	
	(1) ATI_fragment_shader provides a procedural function call
	    interface to specify the fragment program, whereas
	    ATI_text_fragment_shader uses a textual string to specify
	    the program.  The fundamental syntax and constructs of the
	    program "language" remain the same.
	
	(2) The program object managment portions of the interface,
	    namely the routines used to create, bind, and delete program
	    objects and set program constants are managed
	    using the framework defined by ARB_vertex_program.
	
	(3) ATI_fragment_shader refers to the description of the
	    programmable fragment processing as a "fragment shader".
	    In keeping with the desire to treat all programmable parts
	    of the pipeline consistently, ATI_text_fragment_shader refers
	    to these as "fragment programs".  The name of the extension is
	    left as ATI_text_fragment_shader instead of
	    ATI_text_fragment_program in order to indicate the underlying
	    similarity between the API's of the two extensions, and to
	    differentiate it from any other potential extensions that
	    may be able to move even further in the direction of treating
	    fragment programs as just another programmable area of the
	    GL pipeline.
	
	Although ATI_fragment_shader was originally conceived as a
	device-independent extension that would expose the capabilities of
	future generations of hardware, changing trends in programmable
	hardware have affected the lifespan of this extension.  For this
	reason you will now find a fixed set of features and resources
	exposed, and the queries to determine this set have been deprecated
	in ATI_fragment_shader.  Further, in ATI_text_fragment_shader,
	most of these resource limits are fixed by the text grammar and 
	the queries have been removed altogether.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/text_fragment_shader.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.text_fragment_shader import *
from OpenGL.raw.GL.ATI.text_fragment_shader import _EXTENSION_NAME

def glInitTextFragmentShaderATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.EGL_image_storage

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.EGL_image_storage to provide a more 
Python-friendly API

Overview (from the spec)
	
	The OpenGL ES extension OES_EGL_image provides a mechanism for creating
	GL textures sharing storage with EGLImage objects (in other words, creating
	GL texture EGLImage targets).  The extension was written against the
	OpenGL ES 2.0 specification, which does not have the concept of immutable
	textures.  As a result, it specifies that respecification of a texture by
	calling TexImage* on a texture that is an EGLImage target causes it to be
	implicitly orphaned.  In most cases, this is not the desired behavior, but
	rather a result of an application error.
	
	This extension provides a mechanism for creating texture objects that are
	both EGLImage targets and immutable.  Since immutable textures cannot be
	respecified, they also cannot accidentally be orphaned, and attempts to do
	so generate errors instead of resulting in well-defined, but often
	undesirable and surprising behavior.  It provides a strong guarantee that
	texture data that is intended to be shared will remain shared.
	
	EGL extension specifications are located in the EGL Registry at
	
	    http://www.khronos.org/registry/egl/

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/EGL_image_storage.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.EGL_image_storage import *
from OpenGL.raw.GLES2.EXT.EGL_image_storage import _EXTENSION_NAME

def glInitEglImageStorageEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension GREMEDY.frame_terminator

This module customises the behaviour of the 
OpenGL.raw.GL.GREMEDY.frame_terminator to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a mechanism that enables marking the end 
	of render frames within the OpenGL stream.
	
	When debugging or profiling an OpenGL application, the debuggers
	and profilers needs to know when a render frame is ended. This
	is important for frame per second measurements, statistical analysis, 
	marking and clearing stream loggers logs, performance counters 
	sampling and more.
	
	When an application uses off screen buffers, the debugger / profiler
	cannot be guaranteed that the application will call a certain function at
	the end of each off-screen frame (e.g: SwapBuffers / glClear / etc). 
	This extension enables the application to notify the debugger / profiler 
	whenever a render frame is ended.
	
	This extension is mainly useful for debuggers and profilers. It is not 
	expected that standard drivers would implement this extension. The main
	point of having this extension is to allow applications to have a clean 
	way of accessing this functionality only when they are run under the 
	control of a debugger / profiler, without having to recompile or change 
	the application source code.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/GREMEDY/frame_terminator.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.GREMEDY.frame_terminator import *
from OpenGL.raw.GL.GREMEDY.frame_terminator import _EXTENSION_NAME

def glInitFrameTerminatorGREMEDY():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_conservative_raster_pre_snap_triangles'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_conservative_raster_pre_snap_triangles',error_checker=_errors._error_checker)
GL_CONSERVATIVE_RASTER_MODE_NV=_C('GL_CONSERVATIVE_RASTER_MODE_NV',0x954D)
GL_CONSERVATIVE_RASTER_MODE_NV=_C('GL_CONSERVATIVE_RASTER_MODE_NV',0x954D)
GL_CONSERVATIVE_RASTER_MODE_POST_SNAP_NV=_C('GL_CONSERVATIVE_RASTER_MODE_POST_SNAP_NV',0x954E)
GL_CONSERVATIVE_RASTER_MODE_PRE_SNAP_TRIANGLES_NV=_C('GL_CONSERVATIVE_RASTER_MODE_PRE_SNAP_TRIANGLES_NV',0x954F)
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glConservativeRasterParameteriNV(pname,param):pass



'''OpenGL extension IMG.user_clip_plane

This module customises the behaviour of the 
OpenGL.raw.GLES1.IMG.user_clip_plane to provide a more 
Python-friendly API

Overview (from the spec)
	
	The OpenGL ES 1.0 Specification does not support user clip planes. This extension
	adds the support for clip planes. This extension was superceded by OpenGL ES 1.1

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/user_clip_plane.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.IMG.user_clip_plane import *
from OpenGL.raw.GLES1.IMG.user_clip_plane import _EXTENSION_NAME

def glInitUserClipPlaneIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glClipPlanefIMG=wrapper.wrapper(glClipPlanefIMG).setInputArraySize(
    'eqn', 4
)
glClipPlanexIMG=wrapper.wrapper(glClipPlanexIMG).setInputArraySize(
    'eqn', 4
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.fragment_shader_interlock

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.fragment_shader_interlock to provide a more 
Python-friendly API

Overview (from the spec)
	
	In unextended OpenGL 4.5, applications may produce a
	large number of fragment shader invocations that perform loads and
	stores to memory using image uniforms, atomic counter uniforms,
	buffer variables, or pointers. The order in which loads and stores
	to common addresses are performed by different fragment shader
	invocations is largely undefined.  For algorithms that use shader
	writes and touch the same pixels more than once, one or more of the
	following techniques may be required to ensure proper execution ordering:
	
	  * inserting Finish or WaitSync commands to drain the pipeline between
	    different "passes" or "layers";
	
	  * using only atomic memory operations to write to shader memory (which
	    may be relatively slow and limits how memory may be updated); or
	
	  * injecting spin loops into shaders to prevent multiple shader
	    invocations from touching the same memory concurrently.
	
	This extension provides new GLSL built-in functions
	beginInvocationInterlockARB() and endInvocationInterlockARB() that delimit
	a critical section of fragment shader code.  For pairs of shader
	invocations with "overlapping" coverage in a given pixel, the OpenGL
	implementation will guarantee that the critical section of the fragment
	shader will be executed for only one fragment at a time.
	
	There are four different interlock modes supported by this extension,
	which are identified by layout qualifiers.  The qualifiers
	"pixel_interlock_ordered" and "pixel_interlock_unordered" provides mutual
	exclusion in the critical section for any pair of fragments corresponding
	to the same pixel.  When using multisampling, the qualifiers
	"sample_interlock_ordered" and "sample_interlock_unordered" only provide
	mutual exclusion for pairs of fragments that both cover at least one
	common sample in the same pixel; these are recommended for performance if
	shaders use per-sample data structures.
	
	Additionally, when the "pixel_interlock_ordered" or
	"sample_interlock_ordered" layout qualifier is used, the interlock also
	guarantees that the critical section for multiple shader invocations with
	"overlapping" coverage will be executed in the order in which the
	primitives were processed by the GL.  Such a guarantee is useful for
	applications like blending in the fragment shader, where an application
	requires that fragment values to be composited in the framebuffer in
	primitive order.
	
	This extension can be useful for algorithms that need to access per-pixel
	data structures via shader loads and stores.  Such algorithms using this
	extension can access such data structures in the critical section without
	worrying about other invocations for the same pixel accessing the data
	structures concurrently.  Additionally, the ordering guarantees are useful
	for cases where the API ordering of fragments is meaningful.  For example,
	applications may be able to execute programmable blending operations in
	the fragment shader, where the destination buffer is read via image loads
	and the final value is written via image stores.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/fragment_shader_interlock.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.fragment_shader_interlock import *
from OpenGL.raw.GL.ARB.fragment_shader_interlock import _EXTENSION_NAME

def glInitFragmentShaderInterlockARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.geometry_shader

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.geometry_shader to provide a more 
Python-friendly API

Overview (from the spec)
	
	OES_geometry_shader defines a new shader type available to be run on the
	GPU, called a geometry shader. Geometry shaders are run after vertices are
	transformed, but prior to color clamping, flatshading and clipping.
	
	A geometry shader begins with a single primitive (point, line,
	triangle). It can read the attributes of any of the vertices in the
	primitive and use them to generate new primitives. A geometry shader has a
	fixed output primitive type (point, line strip, or triangle strip) and
	emits vertices to define a new primitive. A geometry shader can emit
	multiple disconnected primitives. The primitives emitted by the geometry
	shader are clipped and then processed like an equivalent primitive
	specified by the application.
	
	Furthermore, OES_geometry_shader provides four additional primitive
	types: lines with adjacency, line strips with adjacency, separate
	triangles with adjacency, and triangle strips with adjacency.  Some of the
	vertices specified in these new primitive types are not part of the
	ordinary primitives, instead they represent neighboring vertices that are
	adjacent to the two line segment end points (lines/strips) or the three
	triangle edges (triangles/tstrips). These vertices can be accessed by
	geometry shaders and used to match up the vertices emitted by the geometry
	shader with those of neighboring primitives.
	
	Since geometry shaders expect a specific input primitive type, an error
	will occur if the application presents primitives of a different type.
	For example, if a geometry shader expects points, an error will occur at
	drawing time if a primitive mode of TRIANGLES is specified.
	
	This extension also adds the notion of layered framebuffer attachments
	and framebuffers that can be used in conjunction with geometry shaders
	to allow programs to direct primitives to a face of a cube map or layer
	of a three-dimensional texture or two-dimensional array texture. The
	layer used for rendering can be selected by the geometry shader at run
	time. The framebuffer layer count present in GL 4.x and removed from
	ES 3.1 is restored.
	
	Not all geometry shader implementations have the ability to write the
	point size from a geometry shader. Thus a second extension string and
	shading language enable are provided for implementations which do
	support geometry shader point size.
	
	This extension relies on the OES_shader_io_blocks extension to provide
	the required functionality for declaring input and output blocks and
	interfacing between shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/geometry_shader.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.geometry_shader import *
from OpenGL.raw.GLES2.OES.geometry_shader import _EXTENSION_NAME

def glInitGeometryShaderOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.shader_viewport_layer_array

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_viewport_layer_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	The gl_ViewportIndex and gl_Layer built-in variables were introduced by
	the in OpenGL 4.1. These variables are available in un-extended OpenGL
	only to the geometry shader. When written in the geometry shader, they
	cause geometry to be directed to one of an array of several independent
	viewport rectangles or framebuffer attachment layers, respectively.
	
	In order to use any viewport or attachment layer other than zero, a
	geometry shader must be present. Geometry shaders introduce processing
	overhead and potential performance issues. The AMD_vertex_shader_layer
	and AMD_vertex_shader_viewport_index extensions allowed the gl_Layer
	and gl_ViewportIndex outputs to be written directly from the vertex shader
	with no geometry shader present.
	
	This extension effectively merges the AMD_vertex_shader_layer and
	AMD_vertex_shader_viewport_index extensions together and extends them further
	to allow both outputs to be written from tessellation evaluation shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_viewport_layer_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_viewport_layer_array import *
from OpenGL.raw.GL.ARB.shader_viewport_layer_array import _EXTENSION_NAME

def glInitShaderViewportLayerArrayARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARM.shader_framebuffer_fetch_depth_stencil

This module customises the behaviour of the 
OpenGL.raw.GLES2.ARM.shader_framebuffer_fetch_depth_stencil to provide a more 
Python-friendly API

Overview (from the spec)
	
	Existing extensions, such as EXT_shader_framebuffer_fetch, allow fragment
	shaders to read existing framebuffer color data as input. This enables
	use-cases such as programmable blending, and other operations that may not
	be possible to implement with fixed-function blending.
	
	This extension adds similar capabilities for depth and stencil values.
	
	One use-case for this is soft depth-blending of particles. Normally, this
	would require two render passes: one that writes out the depth values of the
	background geometry to a depth texture, and one that renders the particles
	while reading from the depth texture to do the blending. This extension
	allows this to be done in a single pass.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARM/shader_framebuffer_fetch_depth_stencil.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ARM.shader_framebuffer_fetch_depth_stencil import *
from OpenGL.raw.GLES2.ARM.shader_framebuffer_fetch_depth_stencil import _EXTENSION_NAME

def glInitShaderFramebufferFetchDepthStencilARM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.alpha_test

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.alpha_test to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension reintroduces the alpha test per-fragment operation
	from OpenGL ES 1.x.  Some hardware has a dedicated unit capable of
	performing this operation, and it can save ALU operations in the fragment
	shader by avoiding the conditional discard.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/alpha_test.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.alpha_test import *
from OpenGL.raw.GLES2.QCOM.alpha_test import _EXTENSION_NAME

def glInitAlphaTestQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""ctypes sized data-arrays as a data-formatmechanism

XXX we have to use _ctypes.Array as the type descriminator,
would be nice to have it available from the public module
"""
REGISTRY_NAME = 'ctypesarrays'
import ctypes, _ctypes

from OpenGL.raw.GL import _types
from OpenGL.arrays import _arrayconstants as GL_1_1
from OpenGL import constant
from OpenGL.arrays import formathandler
from OpenGL._bytes import bytes,unicode
import operator

class CtypesArrayHandler( formathandler.FormatHandler ):
    """Ctypes Array-type-specific data-type handler for OpenGL"""
    @classmethod
    def from_param( cls, value, typeCode=None ):
        return ctypes.byref( value )
    dataPointer = staticmethod( ctypes.addressof )
    HANDLED_TYPES = (_ctypes.Array, )
    isOutput = True
    @classmethod
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.byref( value )
    @classmethod
    def zeros( cls, dims, typeCode ):
        """Return Numpy array of zeros in given size"""
        type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
        for dim in dims:
            type *= int(dim)
        return type() # should expicitly set to 0s
    @classmethod
    def ones( cls, dims, typeCode='d' ):
        """Return numpy array of ones in given size"""
        raise NotImplementedError( """Haven't got a good ones implementation yet""" )
##		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
##		for dim in dims:
##			type *= dim 
##		return type() # should expicitly set to 0s
    @classmethod
    def arrayToGLType( cls, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        result = None
        typ = value._type_
        while hasattr(typ,'_type_') and typ not in ARRAY_TO_GL_TYPE_MAPPING:
            typ = typ._type_
        result = ARRAY_TO_GL_TYPE_MAPPING.get( typ )
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                value._type_, list(ARRAY_TO_GL_TYPE_MAPPING.keys()), value,
            )
        )
    @classmethod
    def arraySize( cls, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        try:
            return value.__class__.__component_count__
        except AttributeError as err:
            dims = 1
            for length in cls.dims( value ):
                dims *= length
            value.__class__.__component_count__ = dims
            return dims 
    @classmethod
    def arrayByteCount( cls, value, typeCode = None ):
        """Given a data-value, calculate number of bytes required to represent"""
        return ctypes.sizeof( value )
    @classmethod
    def types( cls, value ):
        """Produce iterable producing all composite types"""
        dimObject = value
        while dimObject is not None:
            yield dimObject
            dimObject = getattr( dimObject, '_type_', None )
            if isinstance( dimObject, (bytes,unicode)):
                dimObject = None 
    @classmethod
    def dims( cls, value ):
        """Produce iterable of all dimensions"""
        try:
            return value.__class__.__dimensions__
        except AttributeError as err:
            dimensions = []
            for base in cls.types( value ):
                length = getattr( base, '_length_', None)
                if length is not None:
                    dimensions.append( length )
            dimensions = tuple( dimensions )
            value.__class__.__dimensions__  = dimensions
            return dimensions
    @classmethod
    def asArray( cls, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        return value
    @classmethod
    def unitSize( cls, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        try:
            return value.__class__.__min_dimension__
        except AttributeError as err:
            dim = cls.dims( value )[-1]
            value.__class__.__min_dimension__ = dim
            return dim
    @classmethod
    def dimensions( cls, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        return tuple( cls.dims(value) )


ARRAY_TO_GL_TYPE_MAPPING = {
    _types.GLdouble: GL_1_1.GL_DOUBLE,
    _types.GLfloat: GL_1_1.GL_FLOAT,
    _types.GLint: GL_1_1.GL_INT,
    _types.GLuint: GL_1_1.GL_UNSIGNED_INT,
    _types.GLshort: GL_1_1.GL_SHORT,
    _types.GLushort: GL_1_1.GL_UNSIGNED_SHORT,
        
    _types.GLchar: GL_1_1.GL_CHAR,
    _types.GLbyte: GL_1_1.GL_BYTE,
    _types.GLubyte: GL_1_1.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    _types.GL_VOID_P: _types.GLvoidp,
    GL_1_1.GL_DOUBLE: _types.GLdouble,
    GL_1_1.GL_FLOAT: _types.GLfloat,
    GL_1_1.GL_INT: _types.GLint,
    GL_1_1.GL_UNSIGNED_INT: _types.GLuint,
    GL_1_1.GL_SHORT: _types.GLshort,
    GL_1_1.GL_UNSIGNED_SHORT: _types.GLushort,
        
    GL_1_1.GL_CHAR: _types.GLchar,
    GL_1_1.GL_BYTE: _types.GLbyte,
    GL_1_1.GL_UNSIGNED_BYTE: _types.GLubyte,
    'f': _types.GLfloat,
    'd': _types.GLdouble,
    'i': _types.GLint,
    'I': _types.GLuint,
    'h': _types.GLshort,
    'H': _types.GLushort,
    'b': _types.GLbyte,
    'B': _types.GLubyte,
    's': _types.GLchar,
}



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_bindable_uniform'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_bindable_uniform',error_checker=_errors._error_checker)
GL_MAX_BINDABLE_UNIFORM_SIZE_EXT=_C('GL_MAX_BINDABLE_UNIFORM_SIZE_EXT',0x8DED)
GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT=_C('GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT',0x8DE3)
GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT=_C('GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT',0x8DE4)
GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT=_C('GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT',0x8DE2)
GL_UNIFORM_BUFFER_BINDING_EXT=_C('GL_UNIFORM_BUFFER_BINDING_EXT',0x8DEF)
GL_UNIFORM_BUFFER_EXT=_C('GL_UNIFORM_BUFFER_EXT',0x8DEE)
@_f
@_p.types(_cs.GLint,_cs.GLuint,_cs.GLint)
def glGetUniformBufferSizeEXT(program,location):pass
@_f
@_p.types(_cs.GLintptr,_cs.GLuint,_cs.GLint)
def glGetUniformOffsetEXT(program,location):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint)
def glUniformBufferEXT(program,location,buffer):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_ES_VERSION_3_2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_ES_VERSION_3_2',error_checker=_errors._error_checker)
GL_BUFFER=_C('GL_BUFFER',0x82E0)
GL_CCW=_C('GL_CCW',0x0901)
GL_CLAMP_TO_BORDER=_C('GL_CLAMP_TO_BORDER',0x812D)
GL_COLORBURN=_C('GL_COLORBURN',0x929A)
GL_COLORDODGE=_C('GL_COLORDODGE',0x9299)
GL_COMPRESSED_RGBA_ASTC_10x10=_C('GL_COMPRESSED_RGBA_ASTC_10x10',0x93BB)
GL_COMPRESSED_RGBA_ASTC_10x5=_C('GL_COMPRESSED_RGBA_ASTC_10x5',0x93B8)
GL_COMPRESSED_RGBA_ASTC_10x6=_C('GL_COMPRESSED_RGBA_ASTC_10x6',0x93B9)
GL_COMPRESSED_RGBA_ASTC_10x8=_C('GL_COMPRESSED_RGBA_ASTC_10x8',0x93BA)
GL_COMPRESSED_RGBA_ASTC_12x10=_C('GL_COMPRESSED_RGBA_ASTC_12x10',0x93BC)
GL_COMPRESSED_RGBA_ASTC_12x12=_C('GL_COMPRESSED_RGBA_ASTC_12x12',0x93BD)
GL_COMPRESSED_RGBA_ASTC_4x4=_C('GL_COMPRESSED_RGBA_ASTC_4x4',0x93B0)
GL_COMPRESSED_RGBA_ASTC_5x4=_C('GL_COMPRESSED_RGBA_ASTC_5x4',0x93B1)
GL_COMPRESSED_RGBA_ASTC_5x5=_C('GL_COMPRESSED_RGBA_ASTC_5x5',0x93B2)
GL_COMPRESSED_RGBA_ASTC_6x5=_C('GL_COMPRESSED_RGBA_ASTC_6x5',0x93B3)
GL_COMPRESSED_RGBA_ASTC_6x6=_C('GL_COMPRESSED_RGBA_ASTC_6x6',0x93B4)
GL_COMPRESSED_RGBA_ASTC_8x5=_C('GL_COMPRESSED_RGBA_ASTC_8x5',0x93B5)
GL_COMPRESSED_RGBA_ASTC_8x6=_C('GL_COMPRESSED_RGBA_ASTC_8x6',0x93B6)
GL_COMPRESSED_RGBA_ASTC_8x8=_C('GL_COMPRESSED_RGBA_ASTC_8x8',0x93B7)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x10=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x10',0x93DB)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x5=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x5',0x93D8)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x6=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x6',0x93D9)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x8=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x8',0x93DA)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x10=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x10',0x93DC)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x12=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x12',0x93DD)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4',0x93D0)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x4=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x4',0x93D1)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x5=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x5',0x93D2)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x5=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x5',0x93D3)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x6=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x6',0x93D4)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x5=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x5',0x93D5)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x6=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x6',0x93D6)
GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x8=_C('GL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x8',0x93D7)
GL_CONTEXT_FLAGS=_C('GL_CONTEXT_FLAGS',0x821E)
GL_CONTEXT_FLAG_DEBUG_BIT=_C('GL_CONTEXT_FLAG_DEBUG_BIT',0x00000002)
GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT=_C('GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT',0x00000004)
GL_CONTEXT_LOST=_C('GL_CONTEXT_LOST',0x0507)
GL_CW=_C('GL_CW',0x0900)
GL_DARKEN=_C('GL_DARKEN',0x9297)
GL_DEBUG_CALLBACK_FUNCTION=_C('GL_DEBUG_CALLBACK_FUNCTION',0x8244)
GL_DEBUG_CALLBACK_USER_PARAM=_C('GL_DEBUG_CALLBACK_USER_PARAM',0x8245)
GL_DEBUG_GROUP_STACK_DEPTH=_C('GL_DEBUG_GROUP_STACK_DEPTH',0x826D)
GL_DEBUG_LOGGED_MESSAGES=_C('GL_DEBUG_LOGGED_MESSAGES',0x9145)
GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH=_C('GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH',0x8243)
GL_DEBUG_OUTPUT=_C('GL_DEBUG_OUTPUT',0x92E0)
GL_DEBUG_OUTPUT_SYNCHRONOUS=_C('GL_DEBUG_OUTPUT_SYNCHRONOUS',0x8242)
GL_DEBUG_SEVERITY_HIGH=_C('GL_DEBUG_SEVERITY_HIGH',0x9146)
GL_DEBUG_SEVERITY_LOW=_C('GL_DEBUG_SEVERITY_LOW',0x9148)
GL_DEBUG_SEVERITY_MEDIUM=_C('GL_DEBUG_SEVERITY_MEDIUM',0x9147)
GL_DEBUG_SEVERITY_NOTIFICATION=_C('GL_DEBUG_SEVERITY_NOTIFICATION',0x826B)
GL_DEBUG_SOURCE_API=_C('GL_DEBUG_SOURCE_API',0x8246)
GL_DEBUG_SOURCE_APPLICATION=_C('GL_DEBUG_SOURCE_APPLICATION',0x824A)
GL_DEBUG_SOURCE_OTHER=_C('GL_DEBUG_SOURCE_OTHER',0x824B)
GL_DEBUG_SOURCE_SHADER_COMPILER=_C('GL_DEBUG_SOURCE_SHADER_COMPILER',0x8248)
GL_DEBUG_SOURCE_THIRD_PARTY=_C('GL_DEBUG_SOURCE_THIRD_PARTY',0x8249)
GL_DEBUG_SOURCE_WINDOW_SYSTEM=_C('GL_DEBUG_SOURCE_WINDOW_SYSTEM',0x8247)
GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR=_C('GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR',0x824D)
GL_DEBUG_TYPE_ERROR=_C('GL_DEBUG_TYPE_ERROR',0x824C)
GL_DEBUG_TYPE_MARKER=_C('GL_DEBUG_TYPE_MARKER',0x8268)
GL_DEBUG_TYPE_OTHER=_C('GL_DEBUG_TYPE_OTHER',0x8251)
GL_DEBUG_TYPE_PERFORMANCE=_C('GL_DEBUG_TYPE_PERFORMANCE',0x8250)
GL_DEBUG_TYPE_POP_GROUP=_C('GL_DEBUG_TYPE_POP_GROUP',0x826A)
GL_DEBUG_TYPE_PORTABILITY=_C('GL_DEBUG_TYPE_PORTABILITY',0x824F)
GL_DEBUG_TYPE_PUSH_GROUP=_C('GL_DEBUG_TYPE_PUSH_GROUP',0x8269)
GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR=_C('GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR',0x824E)
GL_DIFFERENCE=_C('GL_DIFFERENCE',0x929E)
GL_EQUAL=_C('GL_EQUAL',0x0202)
GL_EXCLUSION=_C('GL_EXCLUSION',0x92A0)
GL_FIRST_VERTEX_CONVENTION=_C('GL_FIRST_VERTEX_CONVENTION',0x8E4D)
GL_FRACTIONAL_EVEN=_C('GL_FRACTIONAL_EVEN',0x8E7C)
GL_FRACTIONAL_ODD=_C('GL_FRACTIONAL_ODD',0x8E7B)
GL_FRAGMENT_INTERPOLATION_OFFSET_BITS=_C('GL_FRAGMENT_INTERPOLATION_OFFSET_BITS',0x8E5D)
GL_FRAMEBUFFER_ATTACHMENT_LAYERED=_C('GL_FRAMEBUFFER_ATTACHMENT_LAYERED',0x8DA7)
GL_FRAMEBUFFER_DEFAULT_LAYERS=_C('GL_FRAMEBUFFER_DEFAULT_LAYERS',0x9312)
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS=_C('GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS',0x8DA8)
GL_GEOMETRY_INPUT_TYPE=_C('GL_GEOMETRY_INPUT_TYPE',0x8917)
GL_GEOMETRY_OUTPUT_TYPE=_C('GL_GEOMETRY_OUTPUT_TYPE',0x8918)
GL_GEOMETRY_SHADER=_C('GL_GEOMETRY_SHADER',0x8DD9)
GL_GEOMETRY_SHADER_BIT=_C('GL_GEOMETRY_SHADER_BIT',0x00000004)
GL_GEOMETRY_SHADER_INVOCATIONS=_C('GL_GEOMETRY_SHADER_INVOCATIONS',0x887F)
GL_GEOMETRY_VERTICES_OUT=_C('GL_GEOMETRY_VERTICES_OUT',0x8916)
GL_GUILTY_CONTEXT_RESET=_C('GL_GUILTY_CONTEXT_RESET',0x8253)
GL_HARDLIGHT=_C('GL_HARDLIGHT',0x929B)
GL_HSL_COLOR=_C('GL_HSL_COLOR',0x92AF)
GL_HSL_HUE=_C('GL_HSL_HUE',0x92AD)
GL_HSL_LUMINOSITY=_C('GL_HSL_LUMINOSITY',0x92B0)
GL_HSL_SATURATION=_C('GL_HSL_SATURATION',0x92AE)
GL_IMAGE_BUFFER=_C('GL_IMAGE_BUFFER',0x9051)
GL_IMAGE_CUBE_MAP_ARRAY=_C('GL_IMAGE_CUBE_MAP_ARRAY',0x9054)
GL_INNOCENT_CONTEXT_RESET=_C('GL_INNOCENT_CONTEXT_RESET',0x8254)
GL_INT_IMAGE_BUFFER=_C('GL_INT_IMAGE_BUFFER',0x905C)
GL_INT_IMAGE_CUBE_MAP_ARRAY=_C('GL_INT_IMAGE_CUBE_MAP_ARRAY',0x905F)
GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910C)
GL_INT_SAMPLER_BUFFER=_C('GL_INT_SAMPLER_BUFFER',0x8DD0)
GL_INT_SAMPLER_CUBE_MAP_ARRAY=_C('GL_INT_SAMPLER_CUBE_MAP_ARRAY',0x900E)
GL_ISOLINES=_C('GL_ISOLINES',0x8E7A)
GL_IS_PER_PATCH=_C('GL_IS_PER_PATCH',0x92E7)
GL_LAST_VERTEX_CONVENTION=_C('GL_LAST_VERTEX_CONVENTION',0x8E4E)
GL_LAYER_PROVOKING_VERTEX=_C('GL_LAYER_PROVOKING_VERTEX',0x825E)
GL_LIGHTEN=_C('GL_LIGHTEN',0x9298)
GL_LINES_ADJACENCY=_C('GL_LINES_ADJACENCY',0x000A)
GL_LINE_STRIP_ADJACENCY=_C('GL_LINE_STRIP_ADJACENCY',0x000B)
GL_LOSE_CONTEXT_ON_RESET=_C('GL_LOSE_CONTEXT_ON_RESET',0x8252)
GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS',0x8A32)
GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS',0x8E1E)
GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS',0x8E1F)
GL_MAX_DEBUG_GROUP_STACK_DEPTH=_C('GL_MAX_DEBUG_GROUP_STACK_DEPTH',0x826C)
GL_MAX_DEBUG_LOGGED_MESSAGES=_C('GL_MAX_DEBUG_LOGGED_MESSAGES',0x9144)
GL_MAX_DEBUG_MESSAGE_LENGTH=_C('GL_MAX_DEBUG_MESSAGE_LENGTH',0x9143)
GL_MAX_FRAGMENT_INTERPOLATION_OFFSET=_C('GL_MAX_FRAGMENT_INTERPOLATION_OFFSET',0x8E5C)
GL_MAX_FRAMEBUFFER_LAYERS=_C('GL_MAX_FRAMEBUFFER_LAYERS',0x9317)
GL_MAX_GEOMETRY_ATOMIC_COUNTERS=_C('GL_MAX_GEOMETRY_ATOMIC_COUNTERS',0x92D5)
GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS',0x92CF)
GL_MAX_GEOMETRY_IMAGE_UNIFORMS=_C('GL_MAX_GEOMETRY_IMAGE_UNIFORMS',0x90CD)
GL_MAX_GEOMETRY_INPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_INPUT_COMPONENTS',0x9123)
GL_MAX_GEOMETRY_OUTPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_OUTPUT_COMPONENTS',0x9124)
GL_MAX_GEOMETRY_OUTPUT_VERTICES=_C('GL_MAX_GEOMETRY_OUTPUT_VERTICES',0x8DE0)
GL_MAX_GEOMETRY_SHADER_INVOCATIONS=_C('GL_MAX_GEOMETRY_SHADER_INVOCATIONS',0x8E5A)
GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS=_C('GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS',0x90D7)
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS=_C('GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS',0x8C29)
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS',0x8DE1)
GL_MAX_GEOMETRY_UNIFORM_BLOCKS=_C('GL_MAX_GEOMETRY_UNIFORM_BLOCKS',0x8A2C)
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS=_C('GL_MAX_GEOMETRY_UNIFORM_COMPONENTS',0x8DDF)
GL_MAX_LABEL_LENGTH=_C('GL_MAX_LABEL_LENGTH',0x82E8)
GL_MAX_PATCH_VERTICES=_C('GL_MAX_PATCH_VERTICES',0x8E7D)
GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS=_C('GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS',0x92D3)
GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS',0x92CD)
GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS=_C('GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS',0x90CB)
GL_MAX_TESS_CONTROL_INPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_INPUT_COMPONENTS',0x886C)
GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS',0x8E83)
GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS=_C('GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS',0x90D8)
GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS',0x8E81)
GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS',0x8E85)
GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS=_C('GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS',0x8E89)
GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS=_C('GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS',0x8E7F)
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS=_C('GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS',0x92D4)
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS',0x92CE)
GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS=_C('GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS',0x90CC)
GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS',0x886D)
GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS',0x8E86)
GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS=_C('GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS',0x90D9)
GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS',0x8E82)
GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS=_C('GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS',0x8E8A)
GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS=_C('GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS',0x8E80)
GL_MAX_TESS_GEN_LEVEL=_C('GL_MAX_TESS_GEN_LEVEL',0x8E7E)
GL_MAX_TESS_PATCH_COMPONENTS=_C('GL_MAX_TESS_PATCH_COMPONENTS',0x8E84)
GL_MAX_TEXTURE_BUFFER_SIZE=_C('GL_MAX_TEXTURE_BUFFER_SIZE',0x8C2B)
GL_MIN_FRAGMENT_INTERPOLATION_OFFSET=_C('GL_MIN_FRAGMENT_INTERPOLATION_OFFSET',0x8E5B)
GL_MIN_SAMPLE_SHADING_VALUE=_C('GL_MIN_SAMPLE_SHADING_VALUE',0x8C37)
GL_MULTIPLY=_C('GL_MULTIPLY',0x9294)
GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY=_C('GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY',0x9382)
GL_MULTISAMPLE_LINE_WIDTH_RANGE=_C('GL_MULTISAMPLE_LINE_WIDTH_RANGE',0x9381)
GL_NO_ERROR=_C('GL_NO_ERROR',0)
GL_NO_RESET_NOTIFICATION=_C('GL_NO_RESET_NOTIFICATION',0x8261)
GL_OVERLAY=_C('GL_OVERLAY',0x9296)
GL_PATCHES=_C('GL_PATCHES',0x000E)
GL_PATCH_VERTICES=_C('GL_PATCH_VERTICES',0x8E72)
GL_PRIMITIVES_GENERATED=_C('GL_PRIMITIVES_GENERATED',0x8C87)
GL_PRIMITIVE_BOUNDING_BOX=_C('GL_PRIMITIVE_BOUNDING_BOX',0x92BE)
GL_PRIMITIVE_RESTART_FOR_PATCHES_SUPPORTED=_C('GL_PRIMITIVE_RESTART_FOR_PATCHES_SUPPORTED',0x8221)
GL_PROGRAM=_C('GL_PROGRAM',0x82E2)
GL_PROGRAM_PIPELINE=_C('GL_PROGRAM_PIPELINE',0x82E4)
GL_QUADS=_C('GL_QUADS',0x0007)
GL_QUERY=_C('GL_QUERY',0x82E3)
GL_REFERENCED_BY_GEOMETRY_SHADER=_C('GL_REFERENCED_BY_GEOMETRY_SHADER',0x9309)
GL_REFERENCED_BY_TESS_CONTROL_SHADER=_C('GL_REFERENCED_BY_TESS_CONTROL_SHADER',0x9307)
GL_REFERENCED_BY_TESS_EVALUATION_SHADER=_C('GL_REFERENCED_BY_TESS_EVALUATION_SHADER',0x9308)
GL_RESET_NOTIFICATION_STRATEGY=_C('GL_RESET_NOTIFICATION_STRATEGY',0x8256)
GL_SAMPLER=_C('GL_SAMPLER',0x82E6)
GL_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910B)
GL_SAMPLER_BUFFER=_C('GL_SAMPLER_BUFFER',0x8DC2)
GL_SAMPLER_CUBE_MAP_ARRAY=_C('GL_SAMPLER_CUBE_MAP_ARRAY',0x900C)
GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW=_C('GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW',0x900D)
GL_SAMPLE_SHADING=_C('GL_SAMPLE_SHADING',0x8C36)
GL_SCREEN=_C('GL_SCREEN',0x9295)
GL_SHADER=_C('GL_SHADER',0x82E1)
GL_SOFTLIGHT=_C('GL_SOFTLIGHT',0x929C)
GL_STACK_OVERFLOW=_C('GL_STACK_OVERFLOW',0x0503)
GL_STACK_UNDERFLOW=_C('GL_STACK_UNDERFLOW',0x0504)
GL_STENCIL_INDEX=_C('GL_STENCIL_INDEX',0x1901)
GL_STENCIL_INDEX8=_C('GL_STENCIL_INDEX8',0x8D48)
GL_TESS_CONTROL_OUTPUT_VERTICES=_C('GL_TESS_CONTROL_OUTPUT_VERTICES',0x8E75)
GL_TESS_CONTROL_SHADER=_C('GL_TESS_CONTROL_SHADER',0x8E88)
GL_TESS_CONTROL_SHADER_BIT=_C('GL_TESS_CONTROL_SHADER_BIT',0x00000008)
GL_TESS_EVALUATION_SHADER=_C('GL_TESS_EVALUATION_SHADER',0x8E87)
GL_TESS_EVALUATION_SHADER_BIT=_C('GL_TESS_EVALUATION_SHADER_BIT',0x00000010)
GL_TESS_GEN_MODE=_C('GL_TESS_GEN_MODE',0x8E76)
GL_TESS_GEN_POINT_MODE=_C('GL_TESS_GEN_POINT_MODE',0x8E79)
GL_TESS_GEN_SPACING=_C('GL_TESS_GEN_SPACING',0x8E77)
GL_TESS_GEN_VERTEX_ORDER=_C('GL_TESS_GEN_VERTEX_ORDER',0x8E78)
GL_TEXTURE_2D_MULTISAMPLE_ARRAY=_C('GL_TEXTURE_2D_MULTISAMPLE_ARRAY',0x9102)
GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY=_C('GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY',0x9105)
GL_TEXTURE_BINDING_BUFFER=_C('GL_TEXTURE_BINDING_BUFFER',0x8C2C)
GL_TEXTURE_BINDING_CUBE_MAP_ARRAY=_C('GL_TEXTURE_BINDING_CUBE_MAP_ARRAY',0x900A)
GL_TEXTURE_BORDER_COLOR=_C('GL_TEXTURE_BORDER_COLOR',0x1004)
GL_TEXTURE_BUFFER=_C('GL_TEXTURE_BUFFER',0x8C2A)
GL_TEXTURE_BUFFER_BINDING=_C('GL_TEXTURE_BUFFER_BINDING',0x8C2A)
GL_TEXTURE_BUFFER_DATA_STORE_BINDING=_C('GL_TEXTURE_BUFFER_DATA_STORE_BINDING',0x8C2D)
GL_TEXTURE_BUFFER_OFFSET=_C('GL_TEXTURE_BUFFER_OFFSET',0x919D)
GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT=_C('GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT',0x919F)
GL_TEXTURE_BUFFER_SIZE=_C('GL_TEXTURE_BUFFER_SIZE',0x919E)
GL_TEXTURE_CUBE_MAP_ARRAY=_C('GL_TEXTURE_CUBE_MAP_ARRAY',0x9009)
GL_TRIANGLES=_C('GL_TRIANGLES',0x0004)
GL_TRIANGLES_ADJACENCY=_C('GL_TRIANGLES_ADJACENCY',0x000C)
GL_TRIANGLE_STRIP_ADJACENCY=_C('GL_TRIANGLE_STRIP_ADJACENCY',0x000D)
GL_UNDEFINED_VERTEX=_C('GL_UNDEFINED_VERTEX',0x8260)
GL_UNKNOWN_CONTEXT_RESET=_C('GL_UNKNOWN_CONTEXT_RESET',0x8255)
GL_UNSIGNED_INT_IMAGE_BUFFER=_C('GL_UNSIGNED_INT_IMAGE_BUFFER',0x9067)
GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY=_C('GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY',0x906A)
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910D)
GL_UNSIGNED_INT_SAMPLER_BUFFER=_C('GL_UNSIGNED_INT_SAMPLER_BUFFER',0x8DD8)
GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY=_C('GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY',0x900F)
GL_VERTEX_ARRAY=_C('GL_VERTEX_ARRAY',0x8074)
@_f
@_p.types(None,)
def glBlendBarrier():pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum)
def glBlendEquationSeparatei(buf,modeRGB,modeAlpha):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glBlendEquationi(buf,mode):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glBlendFuncSeparatei(buf,srcRGB,dstRGB,srcAlpha,dstAlpha):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum)
def glBlendFunci(buf,src,dst):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLboolean,_cs.GLboolean,_cs.GLboolean,_cs.GLboolean)
def glColorMaski(index,r,g,b,a):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glCopyImageSubData(srcName,srcTarget,srcLevel,srcX,srcY,srcZ,dstName,dstTarget,dstLevel,dstX,dstY,dstZ,srcWidth,srcHeight,srcDepth):pass
@_f
@_p.types(None,_cs.GLDEBUGPROC,ctypes.c_void_p)
def glDebugMessageCallback(callback,userParam):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,_cs.GLboolean)
def glDebugMessageControl(source,type,severity,count,ids,enabled):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLenum,_cs.GLsizei,arrays.GLcharArray)
def glDebugMessageInsert(source,type,id,severity,length,buf):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDisablei(target,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLint)
def glDrawElementsBaseVertex(mode,count,type,indices,basevertex):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLint)
def glDrawElementsInstancedBaseVertex(mode,count,type,indices,instancecount,basevertex):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLint)
def glDrawRangeElementsBaseVertex(mode,start,end,count,type,indices,basevertex):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEnablei(target,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glFramebufferTexture(target,attachment,texture,level):pass
@_f
@_p.types(_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetDebugMessageLog(count,bufSize,sources,types,ids,severities,lengths,messageLog):pass
@_f
@_p.types(_cs.GLenum,)
def glGetGraphicsResetStatus():pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetObjectLabel(identifier,name,bufSize,length,label):pass
@_f
@_p.types(None,ctypes.c_void_p,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetObjectPtrLabel(ptr,bufSize,length,label):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLvoidpArray)
def glGetPointerv(pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetSamplerParameterIiv(sampler,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLuintArray)
def glGetSamplerParameterIuiv(sampler,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetTexParameterIiv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glGetTexParameterIuiv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glGetnUniformfv(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glGetnUniformiv(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glGetnUniformuiv(program,location,bufSize,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum,_cs.GLuint)
def glIsEnabledi(target,index):pass
@_f
@_p.types(None,_cs.GLfloat)
def glMinSampleShading(value):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLcharArray)
def glObjectLabel(identifier,name,length,label):pass
@_f
@_p.types(None,ctypes.c_void_p,_cs.GLsizei,arrays.GLcharArray)
def glObjectPtrLabel(ptr,length,label):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glPatchParameteri(pname,value):pass
@_f
@_p.types(None,)
def glPopDebugGroup():pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glPrimitiveBoundingBox(minX,minY,minZ,minW,maxX,maxY,maxZ,maxW):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLcharArray)
def glPushDebugGroup(source,id,length,message):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glReadnPixels(x,y,width,height,format,type,bufSize,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glSamplerParameterIiv(sampler,pname,param):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLuintArray)
def glSamplerParameterIuiv(sampler,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glTexBuffer(target,internalformat,buffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glTexBufferRange(target,internalformat,buffer,offset,size):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glTexParameterIiv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLuintArray)
def glTexParameterIuiv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexStorage3DMultisample(target,samples,internalformat,width,height,depth,fixedsamplelocations):pass



'''OpenGL extension OVR.multiview_multisampled_render_to_texture

This module customises the behaviour of the 
OpenGL.raw.GLES2.OVR.multiview_multisampled_render_to_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension brings to multiview rendering the functionality
	originally introduced in EXT_multisampled_render_to_texture.
	Essentially this just means adding one new function for
	multisample multiview array attachments in the framebuffer object.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OVR/multiview_multisampled_render_to_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OVR.multiview_multisampled_render_to_texture import *
from OpenGL.raw.GLES2.OVR.multiview_multisampled_render_to_texture import _EXTENSION_NAME

def glInitMultiviewMultisampledRenderToTextureOVR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.shader_subgroup_partitioned

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.shader_subgroup_partitioned to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables support for the NV_shader_subgroup_partitioned
	shading language extension in OpenGL and OpenGL ES.
	
	This extension adds a new SUBGROUP_FEATURE_PARTITIONED_BIT_NV feature bit
	that is returned by queryies for SUBGROUP_SUPPORTED_FEATURES_KHR.
	
	In OpenGL implementations supporting SPIR-V, this extension enables
	support for the SPV_NV_shader_subgroup_partitioned extension.
	
	In OpenGL ES implementations, this extension does NOT add support for
	SPIR-V or for any of the built-in shading language functions (8.18)
	that have genDType (double) prototypes.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_subgroup_partitioned.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.shader_subgroup_partitioned import *
from OpenGL.raw.GLES2.NV.shader_subgroup_partitioned import _EXTENSION_NAME

def glInitShaderSubgroupPartitionedNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.framebuffer_foveated

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.framebuffer_foveated to provide a more 
Python-friendly API

Overview (from the spec)
	
	Foveated rendering is a technique that aims to reduce fragment processing
	workload and bandwidth by reducing the average resolution of a framebuffer.
	Perceived image quality is kept high by leaving the focal point of
	rendering at full resolution.
	
	It exists in two major forms:
	
	    - Static foveated(lens matched) rendering: where the gaze point is
	    fixed with a large fovea region and designed to match up with the lens
	    characteristics.
	    - Eye-tracked foveated rendering: where the gaze point is continuously
	    tracked by a sensor to allow a smaller fovea region (further reducing
	    average resolution)
	
	Traditionally foveated rendering involves breaking a framebuffer's area
	into smaller regions such as bins, tiles, viewports, or layers which are
	rendered to individually. Each of these regions has the geometry projected
	or scaled differently so that the net resolution of these layers is less
	than the original framebuffer's resolution. When these regions are mapped
	back to the original framebuffer, they create a rendered result with
	decreased quality as pixels get further from the focal point.
	
	Foveated rendering is currently achieved by large modifications to an
	applications render pipelines to manually implement the required geometry
	amplifications, blits, and projection changes.  This presents a large
	implementation cost to an application developer and is generally
	inefficient as it can not make use of a platforms unique hardware features
	or optimized software paths. This extension aims to address these problems
	by exposing foveated rendering in an explicit and vendor neutral way, and by
	providing an interface with minimal changes to how an application specifies
	its framebuffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/framebuffer_foveated.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.framebuffer_foveated import *
from OpenGL.raw.GLES2.QCOM.framebuffer_foveated import _EXTENSION_NAME

def glInitFramebufferFoveatedQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glFramebufferFoveationConfigQCOM=wrapper.wrapper(glFramebufferFoveationConfigQCOM).setInputArraySize(
    'providedFeatures', 1
)
### END AUTOGENERATED SECTION


'''OpenGL extension NVX.conditional_render

This module customises the behaviour of the 
OpenGL.raw.GL.NVX.conditional_render to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for conditional rendering based on the
	results of an occlusion query.  This mechanism allows an application to
	potentially reduce the latency between the completion of an occlusion
	query and the rendering commands depending on its result.  It additionally
	allows the decision of whether to render to be made without application
	intervention.
	
	This extension defines two new functions, BeginConditionalRenderNVX and
	EndConditionalRenderNVX, between which rendering commands may be discarded
	based on the results of an occlusion query.  If the specified occlusion
	query returns a non-zero value, rendering commands between these calls are
	executed.  If the occlusion query returns a value of zero, all rendering
	commands between the calls are discarded.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NVX/conditional_render.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NVX.conditional_render import *
from OpenGL.raw.GL.NVX.conditional_render import _EXTENSION_NAME

def glInitConditionalRenderNVX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension APPLE.color_buffer_packed_float

This module customises the behaviour of the 
OpenGL.raw.GLES2.APPLE.color_buffer_packed_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows two packed floating point formats 
	R11F_G11F_B10F and as RGB9_E5 defined in APPLE_texture_packed_float or
	OpenGL ES 3.0 or to be rendered to via framebuffer objects.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/color_buffer_packed_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.APPLE.color_buffer_packed_float import *
from OpenGL.raw.GLES2.APPLE.color_buffer_packed_float import _EXTENSION_NAME

def glInitColorBufferPackedFloatAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SC.VERSION_2_0

This module customises the behaviour of the 
OpenGL.raw.GLSC2.SC.VERSION_2_0 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SC/VERSION_2_0.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLSC2 import _types, _glgets
from OpenGL.raw.GLSC2.SC.VERSION_2_0 import *
from OpenGL.raw.GLSC2.SC.VERSION_2_0 import _EXTENSION_NAME

def glInitVersion20SC():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glBufferData.data size not checked against size
glBufferData=wrapper.wrapper(glBufferData).setInputArraySize(
    'data', None
)
# INPUT glBufferSubData.data size not checked against size
glBufferSubData=wrapper.wrapper(glBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexSubImage2D.data size not checked against imageSize
glCompressedTexSubImage2D=wrapper.wrapper(glCompressedTexSubImage2D).setInputArraySize(
    'data', None
)
# INPUT glDrawRangeElements.indices size not checked against 'count,type'
glDrawRangeElements=wrapper.wrapper(glDrawRangeElements).setInputArraySize(
    'indices', None
)
glGenBuffers=wrapper.wrapper(glGenBuffers).setOutput(
    'buffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenFramebuffers=wrapper.wrapper(glGenFramebuffers).setOutput(
    'framebuffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenRenderbuffers=wrapper.wrapper(glGenRenderbuffers).setOutput(
    'renderbuffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenTextures=wrapper.wrapper(glGenTextures).setOutput(
    'textures',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGetBooleanv=wrapper.wrapper(glGetBooleanv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetBufferParameteriv=wrapper.wrapper(glGetBufferParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetFloatv=wrapper.wrapper(glGetFloatv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetFramebufferAttachmentParameteriv=wrapper.wrapper(glGetFramebufferAttachmentParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetIntegerv=wrapper.wrapper(glGetIntegerv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetProgramiv=wrapper.wrapper(glGetProgramiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetRenderbufferParameteriv=wrapper.wrapper(glGetRenderbufferParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexParameterfv=wrapper.wrapper(glGetTexParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexParameteriv=wrapper.wrapper(glGetTexParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glGetnUniformfv.params size not checked against bufSize
glGetnUniformfv=wrapper.wrapper(glGetnUniformfv).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformiv.params size not checked against bufSize
glGetnUniformiv=wrapper.wrapper(glGetnUniformiv).setInputArraySize(
    'params', None
)
glGetVertexAttribfv=wrapper.wrapper(glGetVertexAttribfv).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribiv=wrapper.wrapper(glGetVertexAttribiv).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribPointerv=wrapper.wrapper(glGetVertexAttribPointerv).setOutput(
    'pointer',size=(1,),orPassIn=True
)
# INPUT glProgramBinary.binary size not checked against length
glProgramBinary=wrapper.wrapper(glProgramBinary).setInputArraySize(
    'binary', None
)
# INPUT glReadnPixels.data size not checked against bufSize
glReadnPixels=wrapper.wrapper(glReadnPixels).setInputArraySize(
    'data', None
)
# INPUT glTexParameterfv.params size not checked against 'pname'
glTexParameterfv=wrapper.wrapper(glTexParameterfv).setInputArraySize(
    'params', None
)
# INPUT glTexParameteriv.params size not checked against 'pname'
glTexParameteriv=wrapper.wrapper(glTexParameteriv).setInputArraySize(
    'params', None
)
# INPUT glTexSubImage2D.pixels size not checked against 'format,type,width,height'
glTexSubImage2D=wrapper.wrapper(glTexSubImage2D).setInputArraySize(
    'pixels', None
)
# INPUT glUniform1fv.value size not checked against count
glUniform1fv=wrapper.wrapper(glUniform1fv).setInputArraySize(
    'value', None
)
# INPUT glUniform1iv.value size not checked against count
glUniform1iv=wrapper.wrapper(glUniform1iv).setInputArraySize(
    'value', None
)
# INPUT glUniform2fv.value size not checked against count*2
glUniform2fv=wrapper.wrapper(glUniform2fv).setInputArraySize(
    'value', None
)
# INPUT glUniform2iv.value size not checked against count*2
glUniform2iv=wrapper.wrapper(glUniform2iv).setInputArraySize(
    'value', None
)
# INPUT glUniform3fv.value size not checked against count*3
glUniform3fv=wrapper.wrapper(glUniform3fv).setInputArraySize(
    'value', None
)
# INPUT glUniform3iv.value size not checked against count*3
glUniform3iv=wrapper.wrapper(glUniform3iv).setInputArraySize(
    'value', None
)
# INPUT glUniform4fv.value size not checked against count*4
glUniform4fv=wrapper.wrapper(glUniform4fv).setInputArraySize(
    'value', None
)
# INPUT glUniform4iv.value size not checked against count*4
glUniform4iv=wrapper.wrapper(glUniform4iv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix2fv.value size not checked against count*4
glUniformMatrix2fv=wrapper.wrapper(glUniformMatrix2fv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix3fv.value size not checked against count*9
glUniformMatrix3fv=wrapper.wrapper(glUniformMatrix3fv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix4fv.value size not checked against count*16
glUniformMatrix4fv=wrapper.wrapper(glUniformMatrix4fv).setInputArraySize(
    'value', None
)
glVertexAttrib1fv=wrapper.wrapper(glVertexAttrib1fv).setInputArraySize(
    'v', 1
)
glVertexAttrib2fv=wrapper.wrapper(glVertexAttrib2fv).setInputArraySize(
    'v', 2
)
glVertexAttrib3fv=wrapper.wrapper(glVertexAttrib3fv).setInputArraySize(
    'v', 3
)
glVertexAttrib4fv=wrapper.wrapper(glVertexAttrib4fv).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribPointer.pointer size not checked against 'size,type,stride'
glVertexAttribPointer=wrapper.wrapper(glVertexAttribPointer).setInputArraySize(
    'pointer', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.texture_rg

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_rg to provide a more 
Python-friendly API

Overview (from the spec)
	
	Historically one- and two- component textures have been specified in OpenGL
	using the intensity, luminance or luminance-alpha (I/L/LA) formats.
	With the advent of programmable shaders and render-to-texture capabilites
	these legacy formats carry some historical artifacts which are no longer
	useful.
	
	For example, when sampling from such textures, the luminance values
	are replicated across the color components, and the intensity values are
	replicated across both the color and alpha components. This is no
	longer necessary with programmable shaders.
	
	It is also desirable to be able to render to one- and two-
	component format textures using capabilities such as framebuffer
	objects (FBO), but rendering to I/L/LA formats is under-specified
	(specifically how to map R/G/B/A values to I/L/A texture channels).
	
	This extension adds new base internal formats for the one-component RED
	and two-component RG (red green) texture formats as well as sized
	internal formats for fixed-point, floating-point and pure integer texture
	formats. The new texure formats can be used for texturing as well
	as for rendering into with framebuffer objects.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_rg.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_rg import *
from OpenGL.raw.GL.ARB.texture_rg import _EXTENSION_NAME

def glInitTextureRgARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION
from OpenGL import images as _images
_images.COMPONENT_COUNTS.update( {
GL_R16:1,
GL_R16F:1,
GL_R16I:1,
GL_R16UI:1,
GL_R32F:1,
GL_R32I:1,
GL_R32UI:1,
GL_R8:1,
GL_R8I:1,
GL_R8UI:1,

GL_RG:2,
GL_RG16:2,
GL_RG16F:2,
GL_RG16I:2,
GL_RG16UI:2,
GL_RG32F:2,
GL_RG32I:2,
GL_RG32UI:2,
GL_RG8:2,
GL_RG8I:2,
GL_RG8UI:2,
GL_RG_INTEGER:2,
})




'''OpenGL extension KHR.robust_buffer_access_behavior

This module customises the behaviour of the 
OpenGL.raw.GL.KHR.robust_buffer_access_behavior to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension specifies the behavior of out-of-bounds buffer and
	array accesses. This is an improvement over the existing
	KHR_robustness extension which states that the application should
	not crash, but that behavior is otherwise undefined. This extension
	specifies the access protection provided by the GL to ensure that
	out-of-bounds accesses cannot read from or write to data not owned
	by the application. All accesses are contained within the buffer
	object and program area they reference. These additional robustness
	guarantees apply to contexts created with the robust access flag
	set.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/robust_buffer_access_behavior.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.KHR.robust_buffer_access_behavior import *
from OpenGL.raw.GL.KHR.robust_buffer_access_behavior import _EXTENSION_NAME

def glInitRobustBufferAccessBehaviorKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Numpy (new version) module implementation of the OpenGL-ctypes array interfaces

XXX Need to register handlers for all of the scalar types that numpy returns,
would like to have all return values be int/float if they are of  compatible
type as well.
"""
REGISTRY_NAME = 'numpy'
import logging
from OpenGL import _configflags
_log = logging.getLogger( __name__ )
try:
    import numpy
except ImportError as err:
    raise ImportError( """No numpy module present: %s"""%(err))
import OpenGL
assert OpenGL
import ctypes
from OpenGL._bytes import long
from OpenGL.raw.GL import _types 
from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL import error
from OpenGL.arrays import formathandler
c_void_p = ctypes.c_void_p
from OpenGL import acceleratesupport
NumpyHandler = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.numpy_formathandler import NumpyHandler
    except ImportError as err:
        _log.warning(
            "Unable to load numpy_formathandler accelerator from OpenGL_accelerate"
        )
if NumpyHandler is None:
    # numpy's array interface has changed over time :(
    testArray = numpy.array( [1,2,3,4],'i' )
    # Numpy's "ctypes" interface actually creates a new ctypes object
    # in python for every access of the .ctypes attribute... which can take
    # ridiculously large periods when you multiply it by millions of iterations
    if hasattr(testArray,'__array_interface__'):
        def dataPointer( cls, instance ):
            """Convert given instance to a data-pointer value (integer)"""
            try:
                return long(instance.__array_interface__['data'][0])
            except AttributeError:
                instance = cls.asArray( instance )
                try:
                    return long(instance.__array_interface__['data'][0])
                except AttributeError:
                    return long(instance.__array_data__[0],0)
    else:
        def dataPointer( cls, instance ):
            """Convert given instance to a data-pointer value (integer)"""
            try:
                return long(instance.__array_data__[0],0)
            except AttributeError:
                instance = cls.asArray( instance )
                try:
                    return long(instance.__array_interface__['data'][0])
                except AttributeError:
                    return long(instance.__array_data__[0],0)
    try:
        del testArray
    except NameError as err:
        pass
    dataPointer = classmethod( dataPointer )


    class NumpyHandler( formathandler.FormatHandler ):
        """Numpy-specific data-type handler for OpenGL

        Attributes:

            ERROR_ON_COPY -- if True, will raise errors
                if we have to copy an array object in order to produce
                a contiguous array of the correct type.
        """
        HANDLED_TYPES = (
            numpy.ndarray,
            numpy.bool_,
            numpy.intc, 
            numpy.uintc,
            numpy.int8,
            numpy.uint8,
            numpy.int16,
            numpy.uint16,
            numpy.int32,
            numpy.uint32,
            numpy.int64,
            numpy.uint64,
            numpy.int64,
            numpy.uint64,
            numpy.float16,
            numpy.float32,
            numpy.float64,
            numpy.complex64,
            numpy.complex128,
            numpy.bytes_,
            numpy.str_,
            numpy.void,
            numpy.datetime64,
            numpy.timedelta64,
        )# list, tuple )
        if hasattr(numpy,'float128'):
            HANDLED_TYPES += (numpy.float128,)
        if hasattr(numpy,'complex256'):
            HANDLED_TYPES += (numpy.complex256,)
        dataPointer = dataPointer
        isOutput = True
        ERROR_ON_COPY = _configflags.ERROR_ON_COPY
        @classmethod
        def zeros( cls, dims, typeCode ):
            """Return Numpy array of zeros in given size"""
            dims = numpy.array( dims, dtype='i')
            return numpy.zeros( dims, GL_TYPE_TO_ARRAY_MAPPING[typeCode])
        @classmethod
        def arrayToGLType( cls, value ):
            """Given a value, guess OpenGL type of the corresponding pointer"""
            typeCode = value.dtype
            constant = ARRAY_TO_GL_TYPE_MAPPING.get( typeCode )
            if constant is None:
                raise TypeError(
                    """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                        typeCode, list(ARRAY_TO_GL_TYPE_MAPPING.keys()), value,
                    )
                )
            return constant

        @classmethod
        def arraySize( cls, value, typeCode = None ):
            """Given a data-value, calculate dimensions for the array"""
            return value.size
        @classmethod
        def arrayByteCount( cls, value, typeCode = None ):
            """Given a data-value, calculate number of bytes required to represent"""
            try:
                return value.nbytes
            except AttributeError:
                if cls.ERROR_ON_COPY:
                    raise error.CopyError(
                        """Non-numpy array passed to numpy arrayByteCount: %s""",
                        type(value),
                    )
                value = cls.asArray( value, typeCode )
                return value.nbytes
        @classmethod
        def asArray( cls, value, typeCode=None ):
            """Convert given value to an array value of given typeCode"""
            if value is None:
                return value
            else:
                return cls.contiguous( value, typeCode )

        @classmethod
        def contiguous( cls, source, typeCode=None ):
            """Get contiguous array from source

            source -- numpy Python array (or compatible object)
                for use as the data source.  If this is not a contiguous
                array of the given typeCode, a copy will be made,
                otherwise will just be returned unchanged.
            typeCode -- optional 1-character typeCode specifier for
                the numpy.array function.

            All gl*Pointer calls should use contiguous arrays, as non-
            contiguous arrays will be re-copied on every rendering pass.
            Although this doesn't raise an error, it does tend to slow
            down rendering.
            """
            typeCode = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
            try:
                contiguous = source.flags.contiguous
            except AttributeError:
                if typeCode:
                    return numpy.ascontiguousarray( source, typeCode )
                else:
                    return numpy.ascontiguousarray( source )
            else:
                if contiguous and (typeCode is None or typeCode==source.dtype.char):
                    return source
                elif (contiguous and cls.ERROR_ON_COPY):
                    from OpenGL import error
                    raise error.CopyError(
                        """Array of type %r passed, required array of type %r""",
                        source.dtype.char, typeCode,
                    )
                else:
                    # We have to do astype to avoid errors about unsafe conversions
                    # XXX Confirm that this will *always* create a new contiguous array
                    # XXX Guard against wacky conversion types like uint to float, where
                    # we really don't want to have the C-level conversion occur.
                    # XXX ascontiguousarray is apparently now available in numpy!
                    if cls.ERROR_ON_COPY:
                        from OpenGL import error
                        raise error.CopyError(
                            """Non-contiguous array passed""",
                            source,
                        )
                    if typeCode is None:
                        typeCode = source.dtype.char
                    return numpy.ascontiguousarray( source, typeCode )
        @classmethod
        def unitSize( cls, value, typeCode=None ):
            """Determine unit size of an array (if possible)"""
            return value.shape[-1]
        @classmethod
        def dimensions( cls, value, typeCode=None ):
            """Determine dimensions of the passed array value (if possible)"""
            return value.shape
        @classmethod
        def from_param( cls, instance, typeCode=None ):
            try:
                pointer = cls.dataPointer( instance )
            except TypeError:
                array = cls.asArray( instance, typeCode )
                pp = cls.dataPointer( array )
                pp._temporary_array_ = (array,)
                return pp
            else:
                if typeCode and instance.dtype != GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]:
                    raise error.CopyError(
                        """Array of type %r passed, required array of type %r""",
                        instance.dtype.char, typeCode,
                    )
                return c_void_p( pointer )

try:
    numpy.array( [1], 's' )
    SHORT_TYPE = 's'
except TypeError as err:
    SHORT_TYPE = 'h'
    USHORT_TYPE = 'H'

def lookupDtype( char ):
    return numpy.zeros( (1,), dtype=char ).dtype

ARRAY_TO_GL_TYPE_MAPPING = {
    lookupDtype('d'): GL_1_1.GL_DOUBLE,
    lookupDtype('f'): GL_1_1.GL_FLOAT,
    lookupDtype('e'): _types.GL_HALF_FLOAT,
    lookupDtype('i'): GL_1_1.GL_INT,
    lookupDtype(SHORT_TYPE): GL_1_1.GL_SHORT,
    lookupDtype(USHORT_TYPE): GL_1_1.GL_UNSIGNED_SHORT,
    lookupDtype('B'): GL_1_1.GL_UNSIGNED_BYTE,
    lookupDtype('c'): GL_1_1.GL_UNSIGNED_BYTE,
    lookupDtype('b'): GL_1_1.GL_BYTE,
    lookupDtype('I'): GL_1_1.GL_UNSIGNED_INT,
    #lookupDtype('P'), _types.GL_VOID_P, # normally duplicates another type (e.g. 'I')
    None: None,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    GL_1_1.GL_DOUBLE: lookupDtype('d'),
    GL_1_1.GL_FLOAT:lookupDtype('f'),
    _types.GL_HALF_FLOAT: lookupDtype('e'),
    GL_1_1.GL_INT: lookupDtype('i'),
    GL_1_1.GL_BYTE: lookupDtype('b'),
    GL_1_1.GL_SHORT: lookupDtype(SHORT_TYPE),
    GL_1_1.GL_UNSIGNED_INT: lookupDtype('I'),
    GL_1_1.GL_UNSIGNED_BYTE: lookupDtype('B'),
    GL_1_1.GL_UNSIGNED_SHORT: lookupDtype(USHORT_TYPE),
    _types.GL_VOID_P: lookupDtype('P'),
    None: None,
    'e': lookupDtype('e'),
    'f': lookupDtype('f'),
    'd': lookupDtype('d'),
    'i': lookupDtype('i'),
    'I': lookupDtype('I'),
    'h': lookupDtype('h'),
    'H': lookupDtype('H'),
    'b': lookupDtype('b'),
    'B': lookupDtype('B'),
    's': lookupDtype('B'),
}



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_gpu_shader_half_float'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_gpu_shader_half_float',error_checker=_errors._error_checker)
GL_FLOAT16_MAT2_AMD=_C('GL_FLOAT16_MAT2_AMD',0x91C5)
GL_FLOAT16_MAT2x3_AMD=_C('GL_FLOAT16_MAT2x3_AMD',0x91C8)
GL_FLOAT16_MAT2x4_AMD=_C('GL_FLOAT16_MAT2x4_AMD',0x91C9)
GL_FLOAT16_MAT3_AMD=_C('GL_FLOAT16_MAT3_AMD',0x91C6)
GL_FLOAT16_MAT3x2_AMD=_C('GL_FLOAT16_MAT3x2_AMD',0x91CA)
GL_FLOAT16_MAT3x4_AMD=_C('GL_FLOAT16_MAT3x4_AMD',0x91CB)
GL_FLOAT16_MAT4_AMD=_C('GL_FLOAT16_MAT4_AMD',0x91C7)
GL_FLOAT16_MAT4x2_AMD=_C('GL_FLOAT16_MAT4x2_AMD',0x91CC)
GL_FLOAT16_MAT4x3_AMD=_C('GL_FLOAT16_MAT4x3_AMD',0x91CD)
GL_FLOAT16_NV=_C('GL_FLOAT16_NV',0x8FF8)
GL_FLOAT16_VEC2_NV=_C('GL_FLOAT16_VEC2_NV',0x8FF9)
GL_FLOAT16_VEC3_NV=_C('GL_FLOAT16_VEC3_NV',0x8FFA)
GL_FLOAT16_VEC4_NV=_C('GL_FLOAT16_VEC4_NV',0x8FFB)

